@@ -1,8 +1,8 @@
 /* appicon.c- icon for applications (not mini-window)
  *
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 #include "wconfig.h"
@@ -26,7 +27,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "generic/wwmlib.h"
 #include "WindowMaker.h"
 #include "wcore.h"
 #include "window.h"
@@ -38,10 +38,11 @@
 #include "funcs.h"
 #include "defaults.h"
 #include "workspace.h"
-
-#ifdef SUPERFLUOUS
 #include "superfluous.h"
-#endif
+#include "menu.h"
+#include "framewin.h"
+#include "dialog.h"
+#include "client.h"
 
 /* 
  * icon_file for the dock is got from the preferences file by
@@ -52,80 +53,58 @@
 extern Cursor wCursor[WCUR_LAST];
 extern WPreferences wPreferences;
 
-#define MOD_MASK wPreferences.modifier_mask 
+#define MOD_MASK       wPreferences.modifier_mask
 
 void appIconMouseDown(WObjDescriptor *desc, XEvent *event);
 static void iconDblClick(WObjDescriptor *desc, XEvent *event);
 static void iconExpose(WObjDescriptor *desc, XEvent *event);
 
 
-/*
- *---------------------------------------------------------------------- 
- * wAppIconCreateFromIcon--
- * 	Create an icon for use in the Dock, by "converting" a normal
- * icon to a dock icon. Tries to guess how the app is launched and
- * if it can't, fail.
- * 
- * Returns:
- * 	A WAppIcon structure or NULL, if the icon can't be docked.
- * 
- *---------------------------------------------------------------------- 
- */
-WAppIcon*
-wAppIconCreateFromIcon(WIcon *icon)
-{
-    WAppIcon *dicon;
-    char **argv;
-    int argc;
-
-    dicon = wmalloc(sizeof(WAppIcon));
-    memset(dicon, 0, sizeof(WAppIcon));
-    dicon->yindex = -1;
-    dicon->xindex = -1;
-    
-    dicon->icon = icon;
-
-    /* check WM_COMMAND */
-    if (!wApplicationOf(icon->owner->main_window)) {
-	dicon->main_window = icon->owner->client_win;
-    } else {
-	dicon->main_window = icon->owner->main_window;
-    }
-    if (!XGetCommand(dpy, dicon->main_window, &argv, &argc) || !argv) {
-	free(dicon);
-	return NULL;
-    }
-    dicon->command = FlattenStringList(argv, argc);
-    XFreeStringList(argv);
-    dicon->wm_class = strdup(icon->owner->wm_class);
-    dicon->wm_instance = strdup(icon->owner->wm_instance);
-    
-    return dicon;
-}
-
 
 WAppIcon*
-wAppIconCreateForCommand(WScreen *scr, char *command, char *wm_class,
-			 char *wm_instance)
+wAppIconCreateForDock(WScreen *scr, char *command, char *wm_instance,
+		      char *wm_class, int tile)
 {
     WAppIcon *dicon;
+    char *path;
 
     dicon = wmalloc(sizeof(WAppIcon));
+    wretain(dicon);
     memset(dicon, 0, sizeof(WAppIcon));
     dicon->yindex = -1;
     dicon->xindex = -1;
 
-    if (command)
-	dicon->command = strdup(command);
+    dicon->prev = NULL;
+    dicon->next = scr->app_icon_list;
+    if (scr->app_icon_list) {
+        scr->app_icon_list->prev = dicon;
+    }
+    scr->app_icon_list = dicon;
 
+    if (command) {
+	dicon->command = wstrdup(command);	
+    }
     if (wm_class)
-      dicon->wm_class = strdup(wm_class);
+      dicon->wm_class = wstrdup(wm_class);
     if (wm_instance)
-      dicon->wm_instance = strdup(wm_instance);
+      dicon->wm_instance = wstrdup(wm_instance);
 
-    dicon->icon =
-	wIconCreateWithImage(scr, wGetImageForWindowName(scr, wm_class,
-							 wm_instance));
+    path = wDefaultGetIconFile(scr, wm_instance, wm_class, True);
+    if (!path && command) {
+	wApplicationExtractDirPackIcon(scr, command, wm_instance, wm_class);
+
+	path = wDefaultGetIconFile(scr, wm_instance, wm_class, False);
+    }
+    
+    if (path)
+	path = FindImage(wPreferences.icon_path, path);
+
+    dicon->icon = wIconCreateWithIconFile(scr, path, tile);
+    if (path)
+	free(path);
+#ifdef REDUCE_APPICONS
+    dicon->num_apps = 0;
+#endif
 
     /* will be overriden by dock */
     dicon->icon->core->descriptor.handle_mousedown = appIconMouseDown;
@@ -143,16 +122,87 @@ WAppIcon*
 wAppIconCreate(WWindow *leader_win)
 {
     WAppIcon *aicon;
+#ifdef REDUCE_APPICONS
+    WAppIcon *atmp;
+    WAppIconAppList *applist;
+    char *tinstance, *tclass;
+#endif
+    WScreen *scr = leader_win->screen_ptr;
 
     aicon = wmalloc(sizeof(WAppIcon));
+    wretain(aicon);
     memset(aicon, 0, sizeof(WAppIcon));
+#ifdef REDUCE_APPICONS
+    applist = wmalloc(sizeof(WAppIconAppList));
+    memset(applist, 0, sizeof(WAppIconAppList));
+    applist->wapp = wApplicationOf(leader_win->main_window);
+    aicon->applist = applist;
+    if (applist->wapp == NULL) {
+	/* Something's wrong. wApplicationOf() should always return a
+	 * valid structure.  Rather than violate assumptions, bail. -cls
+	 */
+	free(applist);
+	wrelease(aicon);
+	return NULL;
+    }
+#endif
+
     aicon->yindex = -1;
     aicon->xindex = -1;
 
-    aicon->icon = wIconCreate(leader_win, 
-			      wGetImageForWindowName(leader_win->screen_ptr,
-						     leader_win->wm_class,
-						     leader_win->wm_instance));
+    aicon->prev = NULL;
+    aicon->next = scr->app_icon_list;
+    if (scr->app_icon_list) {
+#ifndef REDUCE_APPICONS
+        scr->app_icon_list->prev = aicon;
+#else
+	/* If we aren't going to have a match, jump straight to new appicon */
+	if (leader_win->wm_class == NULL || leader_win->wm_class == NULL)
+	    atmp = NULL;
+	else
+	atmp = scr->app_icon_list;
+
+	while (atmp != NULL) {
+	    if ((tinstance = atmp->wm_instance) == NULL)
+		tinstance = "";
+	    if ((tclass = atmp->wm_class) == NULL)
+		tclass = "";
+	    if ((strcmp(leader_win->wm_class, tclass) == 0) &&
+		(strcmp(leader_win->wm_instance, tinstance) == 0))
+	    {
+		/* We have a winner */
+		wrelease(aicon);	
+		atmp->num_apps++;
+		applist->next = atmp->applist;
+		if (atmp->applist)
+		    atmp->applist->prev = applist;
+		atmp->applist = applist;
+
+		if (atmp->docked) {
+		    wDockSimulateLaunch(atmp->dock, atmp);
+		}
+
+		return atmp;
+	    }
+	    atmp = atmp->next;
+	}
+	if (atmp == NULL) {
+	    scr->app_icon_list->prev = aicon;
+	}
+#endif /* REDUCE_APPICONS */
+    }
+    scr->app_icon_list = aicon;
+
+    if (leader_win->wm_class)
+        aicon->wm_class = wstrdup(leader_win->wm_class);
+    if (leader_win->wm_instance)
+        aicon->wm_instance = wstrdup(leader_win->wm_instance);
+#ifdef REDUCE_APPICONS
+    aicon->num_apps = 1;
+#endif
+
+    aicon->icon = wIconCreate(leader_win);
+
     /* will be overriden if docked */
     aicon->icon->core->descriptor.handle_mousedown = appIconMouseDown;
     aicon->icon->core->descriptor.handle_expose = iconExpose;
@@ -169,6 +219,11 @@ wAppIconCreate(WWindow *leader_win)
 void
 wAppIconDestroy(WAppIcon *aicon)
 {
+    WScreen *scr = aicon->icon->core->screen_ptr;
+#ifdef REDUCE_APPICONS
+    WAppIconAppList *aptmp;
+#endif
+
     RemoveFromStackList(aicon->icon->core);
     wIconDestroy(aicon->icon);
     if (aicon->command)
@@ -178,13 +233,39 @@ wAppIconDestroy(WAppIcon *aicon)
 	free(aicon->dnd_command);
 #endif
     if (aicon->wm_instance)
-      XFree(aicon->wm_instance);
+      free(aicon->wm_instance);
     if (aicon->wm_class)
-      XFree(aicon->wm_class);
+      free(aicon->wm_class);
+#ifdef REDUCE_APPICONS
+    /* There should never be a list but just in case */
+    if (aicon->applist != NULL) {
+	aptmp = aicon->applist;
+    	while (aptmp->next) {
+	    aptmp = aptmp->next;
+	    free(aptmp->prev);
+	}
+	free(aptmp);
+    }
+#endif
 
-    free(aicon);
+    if (aicon == scr->app_icon_list) {
+        if (aicon->next)
+            aicon->next->prev = NULL;
+        scr->app_icon_list = aicon->next;
+    }
+    else {
+        if (aicon->next)
+            aicon->next->prev = aicon->prev;
+        if (aicon->prev)
+            aicon->prev->next = aicon->next;
+    }
+
+    aicon->destroyed = 1;
+    wrelease(aicon);
 }
 
+
+
 #ifdef NEWAPPICON
 static void
 drawCorner(WIcon *icon, WWindow *wwin, int active)
@@ -210,7 +291,15 @@ drawCorner(WIcon *icon, WWindow *wwin, int active)
 #endif /* NEWAPPICON */
 
 
-#ifdef DOCK
+void 
+wAppIconMove(WAppIcon *aicon, int x, int y)
+{
+    XMoveWindow(dpy, aicon->icon->core->window, x, y);
+    aicon->x_pos = x;
+    aicon->y_pos = y;
+}
+
+
 #ifdef WS_INDICATOR
 static void
 updateDockNumbers(WScreen *scr)
@@ -257,7 +346,7 @@ updateDockNumbers(WScreen *scr)
    free(ws_numbers);
 }
 #endif /* WS_INDICATOR */
-#endif /* DOCK */
+
 
 void
 wAppIconPaint(WAppIcon *aicon)
@@ -266,7 +355,7 @@ wAppIconPaint(WAppIcon *aicon)
 
     wIconPaint(aicon->icon);
 
-#ifdef DOCK
+
 # ifdef WS_INDICATOR
     if (aicon->docked && scr->dock && aicon->yindex==0)
 	updateDockNumbers(scr);
@@ -279,7 +368,6 @@ wAppIconPaint(WAppIcon *aicon)
 		  scr->copy_gc, 0, 0, scr->dock_dots->width,
 		  scr->dock_dots->height, 0, 0);
     }
-#endif /* DOCK */
 
 #ifdef NEWAPPICON
     if (!wPreferences.strict_ns && aicon->icon->owner!=NULL) {
@@ -292,7 +380,7 @@ wAppIconPaint(WAppIcon *aicon)
 	    wapp = wApplicationOf(aicon->main_window);
 	    if (!wapp) {
 		active = aicon->icon->owner->flags.focused;
-		wWarning("error in wAppIconPaint(). Please report it");
+		wwarning("error in wAppIconPaint(). Please report it");
 	    } else {
 		active = wapp->main_window_desc->flags.focused;
 		if (wapp->main_window_desc->flags.hidden
@@ -304,6 +392,7 @@ wAppIconPaint(WAppIcon *aicon)
 	  drawCorner(aicon->icon, aicon->icon->owner, active);
     }
 #endif /* NEWAPPICON */
+
     XSetClipMask(dpy, scr->copy_gc, None);
     if (aicon->launching) {
 	XFillRectangle(dpy, aicon->icon->core->window, scr->stipple_gc, 
@@ -315,6 +404,184 @@ wAppIconPaint(WAppIcon *aicon)
 
 #define canBeDocked(wwin)  ((wwin) && ((wwin)->wm_class||(wwin)->wm_instance))
 
+#ifdef REDUCE_APPICONS
+unsigned int
+wAppIconReduceAppCount(WApplication *wapp) 
+{
+    WAppIconAppList *applist;
+
+    if (wapp == NULL)
+	return 0;
+
+    if (wapp->app_icon == NULL) 
+	return 0;
+
+    /* If given a main window, check the applist 
+     * and remove the if it exists 
+     */
+	applist = wapp->app_icon->applist;
+	while (applist != NULL) {
+	        if (applist->wapp == wapp) {
+		    /* If this app owns the appicon, change the appicon's 
+		     * owner to the next app in the list or NULL
+		     */
+	    if (wapp->app_icon->icon->owner == applist->wapp->main_window_desc)
+{
+			if (applist->next) {
+			    wapp->app_icon->icon->owner = applist->next->wapp->main_window_desc;
+			} else if (applist->prev) {
+			    wapp->app_icon->icon->owner = applist->prev->wapp->main_window_desc;
+			} else {
+			    wapp->app_icon->icon->owner = NULL;
+			}
+		    }
+		    if (applist->prev)
+			applist->prev->next = applist->next;
+
+		    if (applist->next)
+			applist->next->prev = applist->prev;
+
+	    if (applist == wapp->app_icon->applist)
+		    	wapp->app_icon->applist = applist->next;
+
+		    free(applist);
+
+		    if (wapp->app_icon->applist != NULL)
+		    	wapp->app_icon->main_window = wapp->app_icon->applist->wapp->main_window;
+
+		    return (--wapp->app_icon->num_apps);
+	    	}
+	    applist = applist->next;
+	}
+    return (--wapp->app_icon->num_apps);
+}
+#endif
+
+
+static void
+hideCallback(WMenu *menu, WMenuEntry *entry)
+{
+    WApplication *wapp = (WApplication*)entry->clientdata;
+    
+    if (wapp->flags.hidden) {
+	wWorkspaceChange(menu->menu->screen_ptr, wapp->last_workspace);
+	wUnhideApplication(wapp, False, False);
+    } else {
+	wHideApplication(wapp);
+    }
+}
+
+
+static void
+unhideHereCallback(WMenu *menu, WMenuEntry *entry)
+{
+    WApplication *wapp = (WApplication*)entry->clientdata;
+    
+    wUnhideApplication(wapp, False, True);
+}
+
+
+static void
+setIconCallback(WMenu *menu, WMenuEntry *entry)
+{
+    WAppIcon *icon = ((WApplication*)entry->clientdata)->app_icon;
+    char *file=NULL;
+    WScreen *scr;
+    int result;
+
+    assert(icon!=NULL);
+
+    if (icon->editing)
+	return;
+    icon->editing = 1;
+    scr = icon->icon->core->screen_ptr;
+    
+    wretain(icon);
+
+    result = wIconChooserDialog(scr, &file);    
+    
+    if (result && !icon->destroyed) {
+	if (file[0]==0) {
+	    free(file);
+	    file = NULL;
+	}
+	if (!wIconChangeImageFile(icon->icon, file)) {
+	    wMessageDialog(scr, _("Error"),
+			   _("Could not open specified icon file"), 
+			   _("OK"), NULL, NULL);
+	} else {
+	    wDefaultChangeIcon(scr, icon->wm_instance, icon->wm_class, file);
+	    wAppIconPaint(icon);
+	}
+	if (file)
+	    free(file);
+    }
+    icon->editing = 0;
+    wrelease(icon);
+}
+
+
+static void
+killCallback(WMenu *menu, WMenuEntry *entry)
+{
+    WApplication *wapp = (WApplication*)entry->clientdata;
+    assert(entry->clientdata!=NULL);
+
+    wretain(wapp->main_window_desc);
+    if (wPreferences.dont_confirm_kill
+	|| wMessageDialog(menu->frame->screen_ptr, _("Kill Application"),
+		       _("This will kill the application.\nAny unsaved changes will be lost.\nPlease confirm."),
+		       _("Yes"), _("No"), NULL)==WAPRDefault) {
+	if (!wapp->main_window_desc->flags.destroyed)
+	    wClientKill(wapp->main_window_desc);
+    }
+    wrelease(wapp->main_window_desc);
+}
+
+
+static WMenu*
+createApplicationMenu(WScreen *scr)
+{
+    WMenu *menu;
+    
+    menu = wMenuCreate(scr, NULL, False);
+    wMenuAddCallback(menu, _("Unhide Here"), unhideHereCallback, NULL);
+    wMenuAddCallback(menu, _("(Un)Hide"), hideCallback, NULL);
+    wMenuAddCallback(menu, _("Set Icon..."), setIconCallback, NULL);
+    wMenuAddCallback(menu, _("Kill"), killCallback, NULL);
+    
+    return menu;
+}
+
+
+static void
+openApplicationMenu(WApplication *wapp, int x, int y)
+{
+    WMenu *menu;
+    WScreen *scr = wapp->main_window_desc->screen_ptr;
+    int i;
+    
+    if (!scr->icon_menu) {
+	scr->icon_menu = createApplicationMenu(scr);
+    }
+
+    menu = scr->icon_menu;
+    if (!menu->flags.realized)
+	wMenuRealize(menu);
+    
+    x -= menu->frame->core->width/2;
+    if (x + menu->frame->core->width > scr->scr_width)
+	x = scr->scr_width - menu->frame->core->width;
+    if (x < 0)
+	x = 0;
+
+    /* set client data */
+    for (i = 0; i < menu->entry_no; i++) {
+	menu->entries[i]->clientdata = wapp;
+    }
+    wMenuMapAt(menu, x, y, False);
+}
+
 
 /******************************************************************/
 
@@ -329,19 +596,44 @@ iconDblClick(WObjDescriptor *desc, XEvent *event)
 {
     WAppIcon *aicon = desc->parent;
     WApplication *wapp;
+    WScreen *scr = aicon->icon->core->screen_ptr;
+    int unhideHere;
 
+#ifndef REDUCE_APPICONS
     assert(aicon->icon->owner!=NULL);
-	
+#else
+    if (aicon->icon->owner == NULL) {
+       fprintf(stderr, "Double-click disabled: missing main window.\n");
+       return;
+    }
+#endif 
+
     wapp = wApplicationOf(aicon->icon->owner->main_window);
-    /* aproveita que ta aqui pra atualizar */
-    wapp->main_window = aicon->icon->owner->main_window;
 #ifdef DEBUG0
     if (!wapp) {
-	wWarning("could not find application descriptor for app icon!!");
+	wwarning("could not find application descriptor for app icon!!");
 	return;
     }
 #endif
-    wUnhideApplication(wapp, event->xbutton.button==Button2);
+#ifdef REDUCE_APPICONS
+    if (!wapp) {
+       fprintf(stderr, "Double-click disabled: missing wapp.\n");
+       return;
+    }
+#endif /* REDUCE_APPICONS */
+#if 0
+    /* aproveita que ta aqui pra atualizar */
+    wapp->main_window = aicon->icon->owner->main_window;
+#endif
+    
+    unhideHere = (event->xbutton.state & ShiftMask);
+    
+    /* go to the last workspace that the user worked on the app */
+    if (!unhideHere)
+	wWorkspaceChange(scr, wapp->last_workspace);
+
+    wUnhideApplication(wapp, event->xbutton.button==Button2, unhideHere);
+    
     if (event->xbutton.state & MOD_MASK) {
 	wHideOtherApplications(aicon->icon->owner);
     }
@@ -358,25 +650,36 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
     int dx=event->xbutton.x, dy=event->xbutton.y;
     int grabbed=0;
     int done=0;
-#ifdef DOCK
+    int superfluous = wPreferences.superfluous; /* we catch it to avoid problems */
     WScreen *scr = icon->core->screen_ptr;
     WWorkspace *workspace = scr->workspaces[scr->current_workspace];
     int shad_x = 0, shad_y = 0, docking=0, dockable, collapsed = 0;
     int ix, iy;
-#ifdef SUPERFLUOUS
+    int clickButton = event->xbutton.button;
     Pixmap ghost = None;
-#endif
-#endif
 
-    if ((desc->click_timestamp>0) 
-	&& (event->xbutton.time - desc->click_timestamp <= DEF_DBLCLICK_TIME)) {
-	desc->click_timestamp = -1;
-	
+    if (aicon->editing)
+	return;
+    
+    if (IsDoubleClick(scr, event)) {
 	iconDblClick(desc, event);
 	return;
     }
-    desc->click_timestamp = event->xbutton.time;
-    
+
+    if (event->xbutton.button == Button3) {
+	WObjDescriptor *desc;
+	WApplication *wapp = wApplicationOf(aicon->icon->owner->main_window);
+
+	openApplicationMenu(wapp, event->xbutton.x_root, 
+			    event->xbutton.y_root);
+
+	/* allow drag select of menu */
+	desc = &scr->icon_menu->menu->descriptor;
+	event->xbutton.send_event = True;
+	(*desc->handle_mousedown)(desc, event);
+	return;
+    }
+
 #ifdef DEBUG
     puts("Moving icon");
 #endif
@@ -386,29 +689,24 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 	wRaiseFrame(icon->core);
 
     
-    /* TODO: remove this send event and make event be passed directly
-     * to the client *//*
-    if (icon->owner && icon->icon_win)
-      XSendEvent(dpy, icon->icon_win, False, ButtonPressMask, event);
-			*/
     if (XGrabPointer(dpy, icon->core->window, True, ButtonMotionMask
-		     |ButtonReleaseMask, GrabModeAsync, GrabModeAsync, None,
-		     None, CurrentTime) !=GrabSuccess) {
-	wWarning("pointer grab failed for appicon move");
+		     |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
+		     GrabModeAsync, None, None, CurrentTime) !=GrabSuccess) {
+	wwarning("pointer grab failed for appicon move");
     }
-#ifdef DOCK
-    if (wPreferences.flags.nodock && wPreferences.flags.nofiend)
+
+    if (wPreferences.flags.nodock && wPreferences.flags.noclip)
       dockable = 0;
     else
       dockable = canBeDocked(icon->owner);
-#endif
+
 
     while (!done) {
-	wMaskEvent(PointerMotionMask|ButtonReleaseMask|ButtonMotionMask
-		   |ExposureMask, &ev);
+	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonPressMask
+		    |ButtonMotionMask|ExposureMask, &ev);
 	switch (ev.type) {
 	 case Expose:
-	    DispatchEvent(&ev);
+	    WMHandleEvent(&ev);
 	    break;
 
 	 case MotionNotify:
@@ -416,7 +714,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		if (abs(dx-ev.xmotion.x)>=MOVE_THRESHOLD
 		    || abs(dy-ev.xmotion.y)>=MOVE_THRESHOLD) {
 		    XChangeActivePointerGrab(dpy, ButtonMotionMask
-					     |ButtonReleaseMask, 
+					    |ButtonReleaseMask|ButtonPressMask, 
 					     wCursor[WCUR_MOVE], CurrentTime);
 		    grabbed=1;
 		} else {
@@ -426,7 +724,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 	    x = ev.xmotion.x_root - dx;
 	    y = ev.xmotion.y_root - dy;
 	    XMoveWindow(dpy, icon->core->window, x, y);
-#ifdef DOCK
+
 	    if (dockable) {
                 if (scr->dock && wDockSnapIcon(scr->dock, aicon, x, y,
                                                &ix, &iy, False)) {
@@ -452,7 +750,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 			wins[0] = icon->core->window;
 			wins[1] = scr->dock_shadow;
 			XRestackWindows(dpy, wins, 2);
-#ifdef SUPERFLUOUS
+                        if (superfluous) {
 			if (icon->pixmap!=None)
 			    ghost = MakeGhostIcon(scr, icon->pixmap);
 			else
@@ -460,27 +758,27 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 			XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
 						   ghost);
 			XClearWindow(dpy, scr->dock_shadow);
-#endif
+                        }
                         XMapWindow(dpy, scr->dock_shadow);
 		    }
 		    docking = 1;
-                } else if (workspace->fiend &&
-                           wDockSnapIcon(workspace->fiend, aicon, x, y,
+                } else if (workspace->clip &&
+                           wDockSnapIcon(workspace->clip, aicon, x, y,
                                          &ix, &iy, False)) {
-                    shad_x = workspace->fiend->x_pos + ix*wPreferences.icon_size;
-		    shad_y = workspace->fiend->y_pos + iy*wPreferences.icon_size;
+                    shad_x = workspace->clip->x_pos + ix*wPreferences.icon_size;
+		    shad_y = workspace->clip->y_pos + iy*wPreferences.icon_size;
 		    
-                    if (scr->last_dock != workspace->fiend && collapsed) {
+                    if (scr->last_dock != workspace->clip && collapsed) {
                         scr->last_dock->collapsed = 1;
                         wDockHideIcons(scr->last_dock);
                         collapsed = 0;
                     }
-                    if (!collapsed && (collapsed = workspace->fiend->collapsed)) {
-                        workspace->fiend->collapsed = 0;
-                        wDockShowIcons(workspace->fiend);
+                    if (!collapsed && (collapsed = workspace->clip->collapsed)) {
+                        workspace->clip->collapsed = 0;
+                        wDockShowIcons(workspace->clip);
                     }
 
-                    scr->last_dock = workspace->fiend;
+                    scr->last_dock = workspace->clip;
 
 		    XMoveWindow(dpy, scr->dock_shadow, shad_x, shad_y);
 		    if (!docking) {
@@ -489,7 +787,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 			wins[0] = icon->core->window;
 			wins[1] = scr->dock_shadow;
 			XRestackWindows(dpy, wins, 2);
-#ifdef SUPERFLUOUS
+                        if (superfluous) {
 			if (icon->pixmap!=None)
 			    ghost = MakeGhostIcon(scr, icon->pixmap);
 			else
@@ -497,7 +795,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 			XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
 						   ghost);
 			XClearWindow(dpy, scr->dock_shadow);
-#endif
+                        }
 			XMapWindow(dpy, scr->dock_shadow);
 		    }
                     docking = 1;
@@ -506,36 +804,50 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
                     docking = 0;
 		}
             }
-#endif /* DOCK */
+
+	    break;
+
+	 case ButtonPress:
 	    break;
 
 	 case ButtonRelease:
+	    if (ev.xbutton.button != clickButton)
+		break;
 	    XUngrabPointer(dpy, CurrentTime);
-#ifdef DOCK
+
             if (docking) {
-		/* icon was docked */
+		Bool docked;
+
+		/* icon is trying to be docked */
 		SlideWindow(icon->core->window, x, y, shad_x, shad_y);
 		XUnmapWindow(dpy, scr->dock_shadow);
-                wDockAttachIcon(scr->last_dock, aicon, ix, iy);
+                docked = wDockAttachIcon(scr->last_dock, aicon, ix, iy);
+                if (scr->last_dock->auto_collapse) {
+                    collapsed = 0;
+                }
+
+		if (!docked) {
+		    /* If icon could not be docked, slide it back to the old
+		     * position */
+		    SlideWindow(icon->core->window, x, y, aicon->x_pos, 
+				aicon->y_pos);
+		}
             } else {
-#endif
 		XMoveWindow(dpy, icon->core->window, x, y);
 		aicon->x_pos = x;
 		aicon->y_pos = y;
-#ifdef DOCK
             }
-            if (collapsed) {
-                scr->last_dock->collapsed = 1;
-                wDockHideIcons(scr->last_dock);
-                collapsed = 0;
-            }
-#ifdef SUPERFLUOUS
-	    if (ghost!=None)
-	      XFreePixmap(dpy, ghost);
-	    XSetWindowBackground(dpy, scr->dock_shadow, scr->white_pixel);
-#endif
-#endif  /* DOCK */
-            done = 1;
+	    if (collapsed) {
+		scr->last_dock->collapsed = 1;
+		wDockHideIcons(scr->last_dock);
+		collapsed = 0;
+	    }
+	    if (superfluous) {
+		if (ghost!=None)
+		    XFreePixmap(dpy, ghost);
+		XSetWindowBackground(dpy, scr->dock_shadow, scr->white_pixel);
+	    }
+	    done = 1;
 	    break;
 	}
     }
