@@ -1,7 +1,7 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 #include "wconfig.h"
 
@@ -30,7 +31,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "generic/wwmlib.h"
 #include "WindowMaker.h"
 #include "wcore.h"
 #include "framewin.h"
@@ -58,6 +58,11 @@ extern Atom _XA_WINDOWMAKER_MENU;
 extern Atom _XA_GNUSTEP_WM_ATTR;
 extern Atom _XA_GNUSTEP_WM_RESIZEBAR;
 
+#ifdef SHAPE
+extern Bool wShapeSupported;
+#endif
+
+
 /*
  *--------------------------------------------------------------------
  * wClientRestore--
@@ -68,16 +73,17 @@ extern Atom _XA_GNUSTEP_WM_RESIZEBAR;
 void
 wClientRestore(WWindow *wwin)
 {
+#if 0
     int gx, gy;
-    
+
     wClientGetGravityOffsets(wwin, &gx, &gy);
     /* set the positio of the frame on screen */
-    wwin->frame_x -= gx * (wwin->old_border_width - FRAME_BORDER_WIDTH);
-    wwin->frame_y -= gy * (wwin->old_border_width - FRAME_BORDER_WIDTH);
+    wwin->frame_x -= gx * FRAME_BORDER_WIDTH;
+    wwin->frame_y -= gy * FRAME_BORDER_WIDTH;
     /* if gravity is to the south, account for the border sizes */
     if (gy > 0)
 	wwin->frame_y += (wwin->frame->top_width + wwin->frame->bottom_width);
-
+#endif
     XUnmapWindow(dpy, wwin->client_win);
     XSetWindowBorderWidth(dpy, wwin->client_win, wwin->old_border_width);    
     XReparentWindow(dpy, wwin->client_win, wwin->screen_ptr->root_win,
@@ -105,7 +111,7 @@ wClientRestore(WWindow *wwin)
 void
 wClientSetState(WWindow *wwin, int state, Window icon_win)
 {
-    unsigned long data[2];
+    CARD32 data[2];
     
     wwin->state = state;
 
@@ -186,7 +192,7 @@ wClientConfigure(WWindow *wwin, XConfigureRequestEvent *xcre)
 	return;
     }
 #ifdef SHAPE
-    {
+    if (wShapeSupported) {
 	int junk;
 	unsigned int ujunk;
 	int b_shaped;
@@ -224,32 +230,35 @@ wClientConfigure(WWindow *wwin, XConfigureRequestEvent *xcre)
 	wwin->old_border_width = xcre->border_width;
     }
 
-    
-    if (xcre->value_mask & CWX)
-      nx = xcre->x - FRAME_BORDER_WIDTH;
-    else
-      nx = wwin->frame_x;
-    
-    if (xcre->value_mask & CWY)
-      ny = xcre->y - FRAME_BORDER_WIDTH - ((ofs_y < 0) ? 0 : wwin->frame->top_width);
-    else
-      ny = wwin->frame_y;
+    if (!wwin->flags.shaded) {
+        /* If the window is shaded, wrong height will be set for the window */
+        if (xcre->value_mask & CWX)
+            nx = xcre->x - FRAME_BORDER_WIDTH;
+        else
+            nx = wwin->frame_x;
 
-    if (xcre->value_mask & CWWidth)
-      nwidth = xcre->width;
-    else
-      nwidth = wwin->frame->core->width;
-    
-    if (xcre->value_mask & CWHeight) 
-      nheight = xcre->height;
-    else
-      nheight = wwin->frame->core->height - wwin->frame->top_width - wwin->frame->bottom_width;
+        if (xcre->value_mask & CWY)
+            ny = xcre->y - ((ofs_y < 0) ? 0 : wwin->frame->top_width)
+		- FRAME_BORDER_WIDTH;
+        else
+            ny = wwin->frame_y;
 
-    wWindowConfigure(wwin, nx, ny, nwidth, nheight);
-    wwin->old_geometry.x = nx;
-    wwin->old_geometry.y = ny;
-    wwin->old_geometry.width = nwidth;
-    wwin->old_geometry.height = nheight;
+        if (xcre->value_mask & CWWidth)
+            nwidth = xcre->width;
+        else
+            nwidth = wwin->frame->core->width;
+
+        if (xcre->value_mask & CWHeight)
+            nheight = xcre->height;
+        else
+            nheight = wwin->frame->core->height - wwin->frame->top_width - wwin->frame->bottom_width;
+
+        wWindowConfigure(wwin, nx, ny, nwidth, nheight);
+        wwin->old_geometry.x = nx;
+        wwin->old_geometry.y = ny;
+        wwin->old_geometry.width = nwidth;
+        wwin->old_geometry.height = nheight;
+    }
 }
 
 
@@ -268,7 +277,7 @@ wClientSendProtocol(WWindow *wwin, Atom protocol, Time time)
     event.xclient.data.l[2] = 0;
     event.xclient.data.l[3] = 0;
     XSendEvent(dpy, wwin->client_win, False, NoEventMask, &event);
-    XFlush(dpy);
+    XSync(dpy, False);
 }
 
 
@@ -304,10 +313,17 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
     switch (event->atom) {
      case XA_WM_NAME:
 	/* window title was changed */
-	
-	if (wwin->frame && MyXFetchName(dpy, wwin->client_win, &tmp)) {
-	    wFrameWindowChangeTitle(wwin->frame, tmp);
-	    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE);
+	if (wwin->frame) {
+	    if (!wFetchName(dpy, wwin->client_win, &tmp)) {
+		/* the hint was removed */
+		tmp = wstrdup(DEF_WINDOW_TITLE);
+	    }
+            if (wFrameWindowChangeTitle(wwin->frame, tmp)) {
+                /* only update the menu if the title has actually changed */
+                UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE);
+            }
+	    if (tmp)
+		free(tmp);
 	}
 	break;
 
@@ -318,7 +334,7 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	    char *new_title;
 	    
 	    /* icon title was changed */
-	    MyXGetIconName(dpy, wwin->client_win, &new_title);
+	    wGetIconName(dpy, wwin->client_win, &new_title);
 	    wIconChangeTitle(wwin->icon, new_title);
 	}
 	break;
@@ -331,10 +347,14 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	    
 	    if (wapp && wapp->app_icon) {
 		if (wapp->app_icon->command!=NULL)
-		  free(wapp->app_icon->command);
+		    free(wapp->app_icon->command);
 
 		if (XGetCommand(dpy, wwin->main_window, &argv, &argc)) {
-		    wapp->app_icon->command = FlattenStringList(argv, argc);
+		    if (argc > 0 && argv != NULL)
+			wapp->app_icon->command = FlattenStringList(argv,argc);
+		    if (argv) {
+			XFreeStringList(argv);
+		    }
 		}
 	    }
 	}
@@ -479,9 +499,16 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	
      case XA_WM_NORMAL_HINTS:
 	/* normal (geometry) hints */
-	
-	XGetWindowAttributes(dpy, wwin->client_win, &attribs);
-	GetNormalHints(wwin, &attribs, False);
+	{
+	    int foo;
+	    unsigned bar;
+
+	    XGetWindowAttributes(dpy, wwin->client_win, &attribs);
+	    wClientGetNormalHints(wwin, &attribs, False, &foo, &foo,
+				  &bar, &bar);
+	    /* TODO: should we check for consistency of the current
+	     * size against the new geometry hints? */
+	}
 	break;
 	
      case XA_WM_TRANSIENT_FOR:
@@ -517,13 +544,15 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		}
 		wwin->transient_for = new_owner;
 		if (new_owner==None) {
-		    if (!wwin->window_flags.miniaturizable) {
-			wwin->window_flags.miniaturizable = 1;
+		    if (wwin->window_flags.no_miniaturizable) {
+			wwin->window_flags.no_miniaturizable = 0;
+			wwin->window_flags.no_miniaturize_button = 0;
 			if (wwin->frame)
 			    wWindowConfigureBorders(wwin);
 		    }
-		} else if (wwin->window_flags.miniaturizable) {
-		    wwin->window_flags.miniaturizable = 0;
+		} else if (!wwin->window_flags.no_miniaturizable) {
+		    wwin->window_flags.no_miniaturizable = 1;
+		    wwin->window_flags.no_miniaturize_button = 1;
 		    if (wwin->frame)
 			wWindowConfigureBorders(wwin);
 		}
@@ -535,9 +564,9 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	if (event->atom==_XA_WM_PROTOCOLS) {
 	    PropGetProtocols(wwin->client_win, &wwin->protocols);
 	    if (wwin->protocols.DELETE_WINDOW) {
-		wwin->window_flags.broken_close = 0;
+		wwin->window_flags.kill_close = 0;
 	    } else {
-		wwin->window_flags.broken_close = 1;		
+		wwin->window_flags.kill_close = 1;		
 	    }
 	    if (wwin->frame)
 		wWindowUpdateButtonImages(wwin);
@@ -545,8 +574,8 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	} else if (event->atom==_XA_WM_COLORMAP_WINDOWS) {
 	    
 	    GetColormapWindows(wwin);
-	    ReInstallActiveColormap();
-	    
+	    wColormapInstallForWindow(wwin->screen_ptr, wwin);
+
 	} else if (event->atom==_XA_WINDOWMAKER_MENU) {
 	    WApplication *wapp;
 
@@ -563,45 +592,49 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		wSetFocusTo(wwin->screen_ptr, 
 			    wwin->screen_ptr->focused_window);
 	    }
-	}
-#if 0
-	else if (event->atom==_XA_GNUSTEP_WM_ATTR) {
+	} else if (event->atom==_XA_GNUSTEP_WM_ATTR) {
 	    GNUstepWMAttributes *attr;
-	    
+
 	    PropGetGNUstepWMAttr(wwin->client_win, &attr);
 	    
+	    wWindowUpdateGNUstepAttr(wwin, attr);
+	    
+	    XFree(attr);
 	}
-#endif
     }
 }
 
 
 /*
  *---------------------------------------------------------------------- 
- * GetNormalHints--
+ * wClientGetNormalHints--
  * 	Get size (normal) hints and a default geometry for the client 
- * window. The hints are also checked for inconsistency.
+ * window. The hints are also checked for inconsistency. If geometry is
+ * True, the returned data will account for client specified initial
+ * geometry.
  * 
  * Side effects:
- * 	normal_hints is filled with valid data and geometry information 
- * about the client is also changed.
+ * 	normal_hints is filled with valid data.
  *---------------------------------------------------------------------- 
  */
 void
-GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
+wClientGetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, Bool geometry,
+		      int *x, int *y, unsigned *width, unsigned *height)
 {
     int pre_icccm=0;
     
     /* find a position for the window */
-    wwin->normal_hints = XAllocSizeHints();
+    if (!wwin->normal_hints)
+        wwin->normal_hints = XAllocSizeHints();
+
     if (!PropGetNormalHints(wwin->client_win, wwin->normal_hints, &pre_icccm)) {
 	wwin->normal_hints->flags = 0;
     }
-    wwin->client.x = wattribs->x;
-    wwin->client.y = wattribs->y;
+    *x = wattribs->x;
+    *y = wattribs->y;
     
-    wwin->client.width = wattribs->width;
-    wwin->client.height = wattribs->height;
+    *width = wattribs->width;
+    *height = wattribs->height;
 
     if (!(wwin->normal_hints->flags & PWinGravity)) {
 	wwin->normal_hints->win_gravity = NorthWestGravity;
@@ -629,6 +662,18 @@ GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
 	  wwin->normal_hints->height_inc = 1;
     }
     
+    if (wwin->normal_hints->flags & PAspect) {
+	if (wwin->normal_hints->min_aspect.x < 1)
+	    wwin->normal_hints->min_aspect.x = 1;
+	if (wwin->normal_hints->min_aspect.y < 1)
+	    wwin->normal_hints->min_aspect.y = 1;
+	
+	if (wwin->normal_hints->max_aspect.x < 1)
+	    wwin->normal_hints->max_aspect.x = 1;
+	if (wwin->normal_hints->max_aspect.y < 1)
+	    wwin->normal_hints->max_aspect.y = 1;
+    }
+    
     if (wwin->normal_hints->min_width <= 0) 
       wwin->normal_hints->min_width = MIN_WINDOW_SIZE;
     
@@ -648,12 +693,12 @@ GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
 	printf("PRE ICCCM\n");	
 #endif	
 	if (wwin->normal_hints->flags & (USPosition|PPosition)) {
-	    wwin->client.x = wwin->normal_hints->x;
-	    wwin->client.y = wwin->normal_hints->y;
+	    *x = wwin->normal_hints->x;
+	    *y = wwin->normal_hints->y;
 	}
 	if (wwin->normal_hints->flags & (USSize|PSize)) {
-	    wwin->client.width = wwin->normal_hints->width;
-	    wwin->client.height = wwin->normal_hints->height;
+	    *width = wwin->normal_hints->width;
+	    *height = wwin->normal_hints->height;
 	}
     }
 }
@@ -665,8 +710,12 @@ GetColormapWindows(WWindow *wwin)
     if (wwin->cmap_windows) {
 	XFree(wwin->cmap_windows);
     }
-    if (!XGetWMColormapWindows(dpy, wwin->client_win, &(wwin->cmap_windows),
-			       &(wwin->cmap_window_no))) {
+
+    wwin->cmap_windows = NULL;
+    wwin->cmap_window_no = 0;
+
+    if (XGetWMColormapWindows(dpy, wwin->client_win, &(wwin->cmap_windows),
+			      &(wwin->cmap_window_no))!=Success) {
 	wwin->cmap_window_no = 0;
 	wwin->cmap_windows = NULL;
     }
