@@ -1,7 +1,7 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,26 +15,23 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 
 #ifndef WMDIALOG_H_
 #define WMDIALOG_H_
 
-#define WD_INFORMATION 	0
-#define WD_ERROR	1
-#define WD_CONFIRMATION	2
-#define WD_EXIT_CONFIRM 3
-#define WD_INPUT	4
 
-#define WDB_OK		(0)
-#define WDB_CANCEL	(1)
-#define WDB_YES		(2)
-#define WDB_NO		(3)
-#define WDB_EXIT	(4)
-
-int wMessageDialog(WScreen *scr, char *title, char *message, int type);
+int wMessageDialog(WScreen *scr, char *title, char *message, 
+		   char *defBtn, char *altBtn, char *othBtn);
 int wInputDialog(WScreen *scr, char *title, char *message, char **text);
 
+Bool wIconChooserDialog(WScreen *scr, char **file);
+
+void wShowInfoPanel(WScreen *scr);
+
+void wShowLegalPanel(WScreen *scr);
+
 #endif
