@@ -1,8 +1,8 @@
 /* icon.c - window icon and dock and appicon parent
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 #include "wconfig.h"
@@ -26,10 +27,11 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 #include <ctype.h>
 #include <wraster.h>
+#include <sys/stat.h>
 
-#include "generic/wwmlib.h"
 #include "WindowMaker.h"
 #include "wcore.h"
 #include "texture.h"
@@ -40,6 +42,7 @@
 #include "stacking.h"
 #include "application.h"
 #include "defaults.h"
+#include "appicon.h"
 
 /**** Global variables ****/
 extern WPreferences wPreferences;
@@ -66,10 +69,13 @@ getSize(Drawable d, unsigned int *w, unsigned int *h, unsigned int *dep)
 
 
 WIcon*
-wIconCreate(WWindow *wwin, RImage *image)
+wIconCreate(WWindow *wwin)
 {
     WScreen *scr=wwin->screen_ptr;
     WIcon *icon;
+    char *file;
+    unsigned long vmask = 0;
+    XSetWindowAttributes attribs;
     
     icon = wmalloc(sizeof(WIcon));
     memset(icon, 0, sizeof(WIcon));
@@ -78,13 +84,16 @@ wIconCreate(WWindow *wwin, RImage *image)
 				     wPreferences.icon_size, 0);
     
     if (wPreferences.use_saveunders) {
-	unsigned long vmask;
-	XSetWindowAttributes attribs;
-	
-	vmask = CWSaveUnder;
+	vmask |= CWSaveUnder;
 	attribs.save_under = True;
-	XChangeWindowAttributes(dpy, icon->core->window, vmask, &attribs);
     }
+    /* a white border for selecting it */
+    vmask |= CWBorderPixel;
+    attribs.border_pixel = scr->white_pixel;
+    
+    XChangeWindowAttributes(dpy, icon->core->window, vmask, &attribs);
+
+    
     /* will be overriden if this is an application icon */
     icon->core->descriptor.handle_mousedown = miniwindowMouseDown;
     icon->core->descriptor.handle_expose = miniwindowExpose;
@@ -94,7 +103,7 @@ wIconCreate(WWindow *wwin, RImage *image)
     icon->core->stacking = wmalloc(sizeof(WStacking));
     icon->core->stacking->above = NULL;
     icon->core->stacking->under = NULL;
-    icon->core->stacking->window_level = NSNormalWindowLevel;
+    icon->core->stacking->window_level = NORMAL_ICON_LEVEL;
     icon->core->stacking->child_of = NULL;
 
     icon->owner = wwin;
@@ -114,11 +123,18 @@ wIconCreate(WWindow *wwin, RImage *image)
 #else
     icon->show_title = 1;
 #endif
-    icon->force_paint = 1;
-    icon->image = image;
-    
-    MyXGetIconName(dpy, wwin->client_win, &icon->icon_name);
+    icon->image = wDefaultGetImage(scr, wwin->wm_instance, wwin->wm_class);
 
+    file = wDefaultGetIconFile(scr, wwin->wm_instance, wwin->wm_class,
+			       False);
+    if (file) {
+        icon->file = wstrdup(file);
+    }
+
+    wGetIconName(dpy, wwin->client_win, &icon->icon_name);
+
+    icon->tile_type = TILE_NORMAL;
+    
     wIconUpdate(icon);
     
     XFlush(dpy);
@@ -128,38 +144,51 @@ wIconCreate(WWindow *wwin, RImage *image)
 
 
 WIcon*
-wIconCreateWithImage(WScreen *scr, RImage *image)
+wIconCreateWithIconFile(WScreen *scr, char *iconfile, int tile)
 {
     WIcon *icon;
+    unsigned long vmask = 0;
+    XSetWindowAttributes attribs;
 
     icon = wmalloc(sizeof(WIcon));
     memset(icon, 0, sizeof(WIcon));
     icon->core = wCoreCreateTopLevel(scr, 0, 0, wPreferences.icon_size, 
 				     wPreferences.icon_size, 0);
     if (wPreferences.use_saveunders) {
-	unsigned long vmask;
-	XSetWindowAttributes attribs;
-	
 	vmask = CWSaveUnder;
 	attribs.save_under = True;
-	XChangeWindowAttributes(dpy, icon->core->window, vmask, &attribs);
     }
+    /* a white border for selecting it */
+    vmask |= CWBorderPixel;
+    attribs.border_pixel = scr->white_pixel;
+	
+    XChangeWindowAttributes(dpy, icon->core->window, vmask, &attribs);
+
     /* will be overriden if this is a application icon */
     icon->core->descriptor.handle_mousedown = miniwindowMouseDown;
     icon->core->descriptor.handle_expose = miniwindowExpose;
     icon->core->descriptor.parent_type = WCLASS_MINIWINDOW;
     icon->core->descriptor.parent = icon;
-    
+
     icon->core->stacking = wmalloc(sizeof(WStacking));
     icon->core->stacking->above = NULL;
     icon->core->stacking->under = NULL;
-    icon->core->stacking->window_level = NSNormalWindowLevel;
+    icon->core->stacking->window_level = NORMAL_ICON_LEVEL;
     icon->core->stacking->child_of = NULL;
 
-    icon->force_paint = 1;
-
-    icon->image = image;
+    if (iconfile) {
+	icon->image = RLoadImage(scr->rcontext, iconfile, 0);
+	if (!icon->image) {
+	    wwarning(_("error loading image file \"%s\""), iconfile, RMessageForError(RErrorCode));
+	}
 
+	icon->image = wIconValidateIconSize(scr, icon->image);
+    
+	icon->file = wstrdup(iconfile);
+    }
+    
+    icon->tile_type = tile;
+    
     wIconUpdate(icon);
 
     return icon;
@@ -173,6 +202,9 @@ wIconDestroy(WIcon *icon)
     WCoreWindow *core = icon->core;
     WScreen *scr = core->screen_ptr;
 
+    if (icon->handlerID)
+	WMDeleteTimerHandler(icon->handlerID);
+
     if (icon->icon_win) {
 	int x=0, y=0;
 
@@ -189,9 +221,12 @@ wIconDestroy(WIcon *icon)
     if (icon->pixmap)
       XFreePixmap(dpy, icon->pixmap);
 
+    if (icon->file)
+        free(icon->file);
+
     if (icon->image!=NULL)
 	RDestroyImage(icon->image);
-    
+
     wCoreDestroy(icon->core);
     free(icon);
 }
@@ -213,15 +248,20 @@ drawIconTitle(WScreen *scr, Pixmap pixmap, int height)
 
 
 static Pixmap
-makeIcon(WScreen *scr, RImage *icon, int titled)
+makeIcon(WScreen *scr, RImage *icon, int titled, int shadowed, int tileType)
 {
     RImage *tile;
     Pixmap pixmap;
-    int x, y, w, h, sx, sy;
+    int x, y, sx, sy;
+    unsigned w, h;
     int theight = scr->icon_title_font->height;
     
-    tile = RCloneImage(scr->icon_tile);
-
+    if (tileType == TILE_NORMAL)
+	tile = RCloneImage(scr->icon_tile);
+    else {
+	assert(scr->clip_tile);
+	tile = RCloneImage(scr->clip_tile);
+    }
     if (icon) {
 	w = (icon->width > wPreferences.icon_size) 
 	    ? wPreferences.icon_size : icon->width;
@@ -236,16 +276,24 @@ makeIcon(WScreen *scr, RImage *icon, int titled)
 	} else {
 	    h = (icon->height+theight > wPreferences.icon_size
 		 ? wPreferences.icon_size-theight : icon->height);
-	    y = theight + (wPreferences.icon_size - icon->height - theight)/2;
+	    y = theight+((int)wPreferences.icon_size-theight-h)/2;
 	    sy = (icon->height - h)/2;
 	}
-	sx=sy=0;
 	RCombineArea(tile, icon, sx, sy, w, h, x, y);
     }
 
-    
+    if (shadowed) {
+        RColor color;
+
+        color.red   = scr->icon_back_texture->light.red   >> 8;
+        color.green = scr->icon_back_texture->light.green >> 8;
+        color.blue  = scr->icon_back_texture->light.blue  >> 8;
+        color.alpha = 150; /* about 60% */
+        RClearImage(tile, &color);
+    }
+
     if (!RConvertImage(scr->rcontext, tile, &pixmap)) {
-	wWarning(_("error rendering image:%s"), RErrorString);
+	wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
     }
     RDestroyImage(tile);
 
@@ -261,37 +309,245 @@ wIconChangeTitle(WIcon *icon, char *new_title)
 {
     int changed;
 
+    changed = (new_title==NULL && icon->icon_name!=NULL)
+	|| (new_title!=NULL && icon->icon_name==NULL);
+
     if (icon->icon_name!=NULL)
 	XFree(icon->icon_name);
-    
-    changed = (new_title==NULL && icon->icon_name!=NULL) 
-	|| (new_title!=NULL && icon->icon_name==NULL);
-    
+
     icon->icon_name = new_title;
 
     if (changed)
-	wIconUpdate(icon);
-    else
-	wIconPaint(icon);
+        icon->force_paint = 1;
+    wIconPaint(icon);
 }
 
 
 void
 wIconChangeImage(WIcon *icon, RImage *new_image)
 {
-    if(!icon)
-        return;
+    assert(icon != NULL);
+    
     if (icon->image)
         RDestroyImage(icon->image);
-    icon->image = new_image;
-    icon->force_paint = 1;
+
+    icon->image = wIconValidateIconSize(icon->core->screen_ptr, new_image);
+
     wIconUpdate(icon);
 }
 
 
+RImage*
+wIconValidateIconSize(WScreen *scr, RImage *icon)
+{
+    RImage *tmp;
+    int w, h;
+    
+    if (!icon)
+	return NULL;
+    
+    if (icon->width > wPreferences.icon_size
+        || icon->height > wPreferences.icon_size) {
+	if (icon->width > icon->height) {
+	    w = wPreferences.icon_size - 4;
+	    h = w*icon->height/icon->width;
+	} else {
+	    h = wPreferences.icon_size - 4;
+	    w = h*icon->width/icon->height;
+	}
+	tmp = RScaleImage(icon, w, h);
+	RDestroyImage(icon);
+	icon = tmp;
+    }
+
+    return icon;
+}
+
+
+Bool
+wIconChangeImageFile(WIcon *icon, char *file)
+{
+    WScreen *scr = icon->core->screen_ptr;
+    RImage *image;
+    char *path;
+    int error = 0;
+
+    if (!file) {
+	wIconChangeImage(icon, NULL);
+	return True;
+    }
+
+    path = FindImage(wPreferences.icon_path, file);
+
+    if (path && (image = RLoadImage(scr->rcontext, path, 0))) {
+	wIconChangeImage(icon, image);
+    } else {
+	error = 1;
+    }
+
+    if (path)
+	free(path);
+
+    return !error;
+}
+
+
+
+static char*
+getnameforicon(WWindow *wwin)
+{
+    char *prefix, *suffix;
+    char *path;
+    int len;
+    
+    if (wwin->wm_class && wwin->wm_instance) {
+	suffix = wmalloc(strlen(wwin->wm_class)+strlen(wwin->wm_instance)+2);
+	sprintf(suffix, "%s.%s", wwin->wm_instance, wwin->wm_class);
+    } else if (wwin->wm_class) {
+	suffix = wmalloc(strlen(wwin->wm_class)+1);
+	strcpy(suffix, wwin->wm_class);
+    } else {
+	suffix = wmalloc(strlen(wwin->wm_instance)+1);
+	strcpy(suffix, wwin->wm_instance);
+    }
+    
+    prefix = getenv("GNUSTEP_USER_PATH");
+    if (!prefix) {
+	len = strlen(wgethomedir())+64+strlen(suffix);
+	path = wmalloc(len+1);
+	sprintf(path, "%s/GNUstep/.AppInfo", wgethomedir());
+    } else {
+	prefix = wexpandpath(prefix);
+	len = strlen(prefix)+64+strlen(suffix);
+	path = wmalloc(len+1);
+	sprintf(path, "%s/.AppInfo", prefix);
+	free(prefix);
+    }
+    if (access(path, F_OK)!=0) {
+	if (mkdir(path, S_IRUSR|S_IWUSR|S_IXUSR)) {
+	    wsyserror(_("could not create directory %s"), path);
+	    free(path);
+	    free(suffix);
+	    return NULL;
+	}
+    }
+    strcat(path, "/WindowMaker");
+    if (access(path, F_OK)!=0) {
+	if (mkdir(path, S_IRUSR|S_IWUSR|S_IXUSR)!=0) {
+	    wsyserror(_("could not create directory %s"), path);
+	    free(path);
+	    free(suffix);
+	    return NULL;
+	}
+    }
+
+    strcat(path, "/");
+    strcat(path, suffix);
+    strcat(path, ".xpm");
+    free(suffix);
+
+    return path;
+}
+
+
+/*
+ * wIconStore--
+ * 	Stores the client supplied icon at ~/GNUstep/.AppInfo/WindowMaker
+ * and returns the path for that icon. Returns NULL if there is no
+ * client supplied icon or on failure.
+ * 
+ * Side effects:
+ * 	New directories might be created.
+ */
+char*
+wIconStore(WIcon *icon)
+{
+    char *path;
+    RImage *image;
+    WWindow *wwin = icon->owner;
+
+    if (!wwin || !wwin->wm_hints || !(wwin->wm_hints->flags & IconPixmapHint)
+	|| wwin->wm_hints->icon_pixmap == None)
+	return NULL;
+
+    path = getnameforicon(wwin);
+
+    image = RCreateImageFromDrawable(icon->core->screen_ptr->rcontext,
+				     wwin->wm_hints->icon_pixmap,
+				     (wwin->wm_hints->flags & IconMaskHint)
+				     ? wwin->wm_hints->icon_mask : None);
+    if (!image) {
+	free(path);
+	return NULL;
+    }
+
+    if (!RSaveImage(image, path, "XPM")) {
+	free(path);
+	path = NULL;
+    }
+    RDestroyImage(image);
+
+    return path;
+}
+
+
+/*
 void
 wIconChangeIconWindow(WIcon *icon, Window new_window);
+*/
 
+static void 
+cycleColor(void *data)
+{
+    WIcon *icon = (WIcon*)data;
+    WScreen *scr = icon->core->screen_ptr;
+    XGCValues gcv;
+
+    icon->step--;
+    gcv.dash_offset = icon->step;
+    XChangeGC(dpy, scr->icon_select_gc, GCDashOffset, &gcv);
+
+    XDrawRectangle(dpy, icon->core->window, scr->icon_select_gc, 0, 0,
+		   icon->core->width-1, icon->core->height-1);
+    icon->handlerID = WMAddTimerHandler(COLOR_CYCLE_DELAY, cycleColor, icon);
+}
+
+
+void
+wIconSetHighlited(WIcon *icon, Bool flag)
+{
+    if (icon->highlighted == flag) {
+	return;
+    }
+    
+    icon->highlighted = flag;
+    wIconPaint(icon);
+}
+
+
+
+void
+wIconSelect(WIcon *icon)
+{
+    WScreen *scr = icon->core->screen_ptr;
+    icon->selected = !icon->selected;
+
+    if (icon->selected) {
+        icon->step = 0;
+	if (!wPreferences.dont_blink)
+            icon->handlerID = WMAddTimerHandler(10, cycleColor, icon);
+        else
+            XDrawRectangle(dpy, icon->core->window, scr->icon_select_gc, 0, 0,
+                           icon->core->width-1, icon->core->height-1);
+    } else {
+        if (icon->handlerID) {
+            WMDeleteTimerHandler(icon->handlerID);
+            icon->handlerID = NULL;
+        }
+	XClearArea(dpy, icon->core->window, 0, 0, icon->core->width,
+		   icon->core->height, True);
+    }
+}
 
 
 void
@@ -313,6 +569,7 @@ wIconUpdate(WIcon *icon)
     
     /* use client specified icon window */
     if (icon->icon_win!=None) {
+	XWindowAttributes attr;
 	int resize=0;
 	int width, height, depth;
 	int theight;
@@ -357,6 +614,13 @@ wIconUpdate(WIcon *icon)
 	
 	icon->pixmap = pixmap;
 
+	if (XGetWindowAttributes(dpy, icon->icon_win, &attr)) {
+	    if (attr.all_event_masks & ButtonPressMask) {
+		wHackedGrabButton(Button1, MOD_MASK, icon->core->window, True, 
+				  ButtonPressMask, GrabModeSync, GrabModeAsync,
+				  None, wCursor[WCUR_ARROW]);
+	    }
+	}
     } else if (wwin && wwin->wm_hints
 	       && (wwin->wm_hints->flags & IconPixmapHint)) {
 	int x, y;
@@ -386,11 +650,12 @@ wIconUpdate(WIcon *icon)
 	    drawIconTitle(scr, pixmap, title_height);
 	    dotitle = 1;
 	    
-	    if (h > wPreferences.icon_size-title_height-2)
+	    if (h > wPreferences.icon_size - title_height - 2) {
 		h = wPreferences.icon_size - title_height - 2;
-
-	    y = (wPreferences.icon_size-h-title_height)/2 + title_height + 1;
-	    
+		y = title_height + 1;
+	    } else {
+		y = (wPreferences.icon_size-h-title_height)/2+title_height + 1;
+	    }
 	} else {
 	    dotitle = 0;
 	    if (w > wPreferences.icon_size)
@@ -404,8 +669,8 @@ wIconUpdate(WIcon *icon)
 	XSetClipOrigin(dpy, scr->copy_gc, x, y);
 	
 	if (d != scr->w_depth) {
-	    XSetForeground(dpy, scr->copy_gc, scr->white_pixel);
-	    XSetBackground(dpy, scr->copy_gc, scr->black_pixel);
+	    XSetForeground(dpy, scr->copy_gc, scr->black_pixel);
+	    XSetBackground(dpy, scr->copy_gc, scr->white_pixel);
 	    XCopyPlane(dpy, wwin->wm_hints->icon_pixmap, pixmap, scr->copy_gc,
 		      0, 0, w, h, x, y, 1);
 	} else {
@@ -418,51 +683,56 @@ wIconUpdate(WIcon *icon)
 	icon->pixmap = pixmap;
     } else {
       user_icon:
-	
-	if (icon->image) {
-	    icon->pixmap = makeIcon(scr, icon->image, icon->show_title);
-	} else {
-	    
-	    /* make default icons */
-	    
-	    if (!scr->def_icon_pixmap) {
-		RImage *image = NULL;
-		char *path;
-		char *file;
 
-		file = wDefaultGetIconFile(scr, NULL, NULL);
-		if (file) {
-		    path = FindImage(wPreferences.pixmap_path, file);
-		    if (!path) {
-			wWarning(_("could not find default icon \"%s\""),file);
-			goto make_icons;
-		    }
-		    free(path);
+          if (icon->image) {
+              icon->pixmap = makeIcon(scr, icon->image, icon->show_title,
+				      icon->shadowed, icon->tile_type);
+          } else {
+              /* make default icons */
 
-		    image = RLoadImage(scr->rcontext, path, 0);
-		    if (!image) {
-			wWarning(_("could not load default icon \"%s\""),file);
-		    }
-		}
+              if (!scr->def_icon_pixmap) {
+                  RImage *image = NULL;
+                  char *path;
+                  char *file;
+
+                  file = wDefaultGetIconFile(scr, NULL, NULL, False);
+                  if (file) {
+                      path = FindImage(wPreferences.icon_path, file);
+                      if (!path) {
+                          wwarning(_("could not find default icon \"%s\""),file);
+                          goto make_icons;
+                      }
+
+                      image = RLoadImage(scr->rcontext, path, 0);
+                      if (!image) {
+                          wwarning(_("could not load default icon \"%s\":%s"),
+				   file, RMessageForError(RErrorCode));
+                      }
+                      free(path);
+                  }
 	make_icons:
-		scr->def_icon_pixmap = makeIcon(scr, image, False);
-		scr->def_ticon_pixmap = makeIcon(scr, image, True);
-		if (image)
-		  RDestroyImage(image);
-	    }	    
+		  
+		  image = wIconValidateIconSize(scr, image);
+                  scr->def_icon_pixmap = makeIcon(scr, image, False, False,
+						  icon->tile_type);
+                  scr->def_ticon_pixmap = makeIcon(scr, image, True, False,
+						   icon->tile_type);
+		  if (image)
+		      RDestroyImage(image);
+              }
 
-	    if (icon->show_title) {
-		XSetWindowBackgroundPixmap(dpy, icon->core->window,
-					   scr->def_ticon_pixmap);
-	    } else {
-		XSetWindowBackgroundPixmap(dpy, icon->core->window,
-					   scr->def_icon_pixmap);
-	    }
-	    icon->pixmap = None;
-	}
+              if (icon->show_title) {
+                  XSetWindowBackgroundPixmap(dpy, icon->core->window,
+                                             scr->def_ticon_pixmap);
+              } else {
+                  XSetWindowBackgroundPixmap(dpy, icon->core->window,
+                                             scr->def_icon_pixmap);
+              }
+              icon->pixmap = None;
+          }
     }
     if (icon->pixmap != None) {
-	XSetWindowBackgroundPixmap(dpy, icon->core->window, icon->pixmap);
+        XSetWindowBackgroundPixmap(dpy, icon->core->window, icon->pixmap);
     }
     XClearWindow(dpy, icon->core->window);
 
@@ -477,11 +747,13 @@ wIconPaint(WIcon *icon)
     WScreen *scr=icon->core->screen_ptr;
     GC gc = scr->icon_title_gc;
     int x;
-#ifndef I18N_MB
     char *tmp;
-#endif
-/*	wIconUpdate(icon);
-*/
+
+    if (icon->force_paint) {
+        icon->force_paint = 0;
+        wIconUpdate(icon);
+        return;
+    }
     
     XClearWindow(dpy, icon->core->window);
     
@@ -489,28 +761,24 @@ wIconPaint(WIcon *icon)
     if (icon->show_title && icon->icon_name!=NULL) {
 	int l;
 	int w;
-#ifdef I18N_MB
-	l = strlen(icon->icon_name);
-	w = MyTextWidth(scr->icon_title_font->font,icon->icon_name,l);
-#else
+
 	tmp = ShrinkString(scr->icon_title_font, icon->icon_name, 
 			   wPreferences.icon_size-4);
-	w = MyTextWidth(scr->icon_title_font->font, tmp, l=strlen(tmp));
-#endif
+	w = wTextWidth(scr->icon_title_font->font, tmp, l=strlen(tmp));
+
 	if (w > icon->core->width - 4)
 	  x = (icon->core->width - 4) - w;
 	else
 	  x = (icon->core->width - w)/2;
 
-#ifdef I18N_MB
- 	XmbDrawString(dpy, icon->core->window, scr->icon_title_font->font, gc, 
- 		      x, 1 + scr->icon_title_font->y, icon->icon_name, l);
-#else
-  	XDrawString(dpy, icon->core->window, gc, x, 
-  		    1 + scr->icon_title_font->y, tmp, l);
+ 	wDrawString(icon->core->window, scr->icon_title_font, gc, 
+ 		      x, 1 + scr->icon_title_font->y, tmp, l);
 	free(tmp);
-#endif
     }
+
+    if (icon->selected)
+	XDrawRectangle(dpy, icon->core->window, scr->icon_select_gc, 0, 0,
+		       icon->core->width-1, icon->core->height-1);
 }
 
 
@@ -543,40 +811,41 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
     int x=wwin->icon_x, y=wwin->icon_y;
     int dx=event->xbutton.x, dy=event->xbutton.y;
     int grabbed=0;
+    int clickButton=event->xbutton.button;
 
     
-    if ((desc->click_timestamp>0) 
-	&& (event->xbutton.time - desc->click_timestamp <= DEF_DBLCLICK_TIME)) {
-	desc->click_timestamp = -1;
-
+    if (IsDoubleClick(icon->core->screen_ptr, event)) {
 	miniwindowDblClick(desc, event);
 	return;
     }
-    desc->click_timestamp = event->xbutton.time;
     
 #ifdef DEBUG
     puts("Moving miniwindow");
 #endif
-    if (event->xbutton.button==wLeftBtn) {
+    if (event->xbutton.button==Button1) {	
 	if (event->xbutton.state & MOD_MASK)
 	    wLowerFrame(icon->core);
 	else
 	    wRaiseFrame(icon->core);
+        if (event->xbutton.state & ShiftMask) {
+            wIconSelect(icon);
+            wSelectWindow(icon->owner, !wwin->flags.selected);
+        }
     }
 
     if (XGrabPointer(dpy, icon->core->window, False, ButtonMotionMask
-		     |ButtonReleaseMask, GrabModeAsync, GrabModeAsync, None, 
-		     None, CurrentTime) !=GrabSuccess) {
+		     |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
+		     GrabModeAsync, None, None, CurrentTime) !=GrabSuccess) {
 #ifdef DEBUG0
-	wWarning("pointer grab failed for icon move");
+	wwarning("pointer grab failed for icon move");
 #endif
     }
     while(1) {
-	wMaskEvent(PointerMotionMask|ButtonReleaseMask|ButtonMotionMask
-		   |ExposureMask, &ev);
+	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonPressMask
+		   |ButtonMotionMask|ExposureMask, &ev);
 	switch (ev.type) {
 	 case Expose:
-	    DispatchEvent(&ev);
+	    WMHandleEvent(&ev);
 	    break;
 
 	 case MotionNotify:
@@ -584,7 +853,7 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 		if (abs(dx-ev.xmotion.x)>=MOVE_THRESHOLD
 		    || abs(dy-ev.xmotion.y)>=MOVE_THRESHOLD) {
 		    XChangeActivePointerGrab(dpy, ButtonMotionMask
-					     |ButtonReleaseMask, 
+					    |ButtonReleaseMask|ButtonPressMask, 
 					     wCursor[WCUR_MOVE], CurrentTime);
 		    grabbed=1;
 		} else {
@@ -596,7 +865,13 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 	    XMoveWindow(dpy, icon->core->window, x, y);
 	    break;
 
+	 case ButtonPress:
+	    break;
+
 	 case ButtonRelease:
+	    if (ev.xbutton.button != clickButton)
+		break;
+
 	    if (wwin->icon_x!=x || wwin->icon_y!=y)
 	      wwin->flags.icon_moved = 1;
 
@@ -612,8 +887,6 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 	    
 	}
     }
-    if (wPreferences.auto_arrange_icons) {
-        wArrangeIcons(wwin->screen_ptr);
-    }
 }
-    
+
+
