@@ -1,7 +1,7 @@
 /* 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 #include "wconfig.h"
 
@@ -29,18 +30,23 @@
 #include <unistd.h>
 #include <pwd.h>
 #include <math.h>
+#include <time.h>
 
 #include <wraster.h>
 
-#include "generic/wwmlib.h"
 #include "WindowMaker.h"
 #include "GNUstep.h"
 #include "screen.h"
 #include "wcore.h"
 #include "window.h"
+#include "framewin.h"
 #include "funcs.h"
 #include "defaults.h"
 #include "dialog.h"
+#include "xutil.h"
+#include "xmodifier.h"
+
+#include "list.h"
 
 /**** global variables *****/
 
@@ -62,7 +68,7 @@ static void
 putdef(char *line, char *name, char *value)
 {
     if (!value) {
-	wWarning(_("could not define value for %s for cpp"), name);
+	wwarning(_("could not define value for %s for cpp"), name);
 	return;
     }
     strcat(line, name);
@@ -92,7 +98,7 @@ username()
 
 	user = getpwuid(getuid());
 	if (!user) {
-	    wSysError(_("could not get password entry for UID %i"), getuid());
+	    wsyserror(_("could not get password entry for UID %i"), getuid());
 	    return NULL;
 	}
 	if (!user->pw_name) {
@@ -116,13 +122,13 @@ MakeCPPArgs(char *path)
     i=1;
     if ((buf=getenv("HOSTNAME"))!=NULL) {
 	if (buf[0]=='(') {
-	    wWarning(_("your machine is misconfigured. HOSTNAME is set to %s"),
+	    wwarning(_("your machine is misconfigured. HOSTNAME is set to %s"),
 		     buf);
 	} else 
 	  putdef(line, " -DHOST=", buf);
     } else if ((buf=getenv("HOST"))!=NULL) {
 	if (buf[0]=='(') {
-	    wWarning(_("your machine is misconfigured. HOST is set to %s"),
+	    wwarning(_("your machine is misconfigured. HOST is set to %s"),
 		     buf);
 	} else 
 	  putdef(line, " -DHOST=", buf);
@@ -163,9 +169,9 @@ MakeCPPArgs(char *path)
       if (buf[0]!='~') {
 	strcpy(fullpath, buf);
       } else {
-	char * gethomedir();
+	char * wgethomedir();
 	/* home is statically allocated. Don't free it! */
-	char *home = gethomedir();
+	char *home = wgethomedir();
 	
 	strcpy(fullpath, home);
 	strcat(fullpath, &(buf[1]));
@@ -185,9 +191,6 @@ MakeCPPArgs(char *path)
 #endif /* USECPP */
 
 
-#define canFocusShaded(win) (win->flags.shaded && \
-                             !(win->flags.miniaturized || win->flags.hidden))
-
 
 WWindow*
 NextFocusWindow(WScreen *scr)
@@ -202,9 +205,9 @@ NextFocusWindow(WScreen *scr)
     min = wwin;
     d = 0xffffffff;
     while (tmp) {
-        if ((tmp->flags.mapped || canFocusShaded(tmp))
-            && tmp->window_flags.focusable &&
-            !tmp->window_flags.skip_window_list) {
+        if (wWindowCanReceiveFocus(tmp) 
+	    && (!tmp->window_flags.skip_window_list
+		|| tmp->flags.internal_window)) {
 	    if (min->client_win > tmp->client_win)
 	      min = tmp;
 	    if (tmp->client_win > wwin->client_win
@@ -235,9 +238,9 @@ PrevFocusWindow(WScreen *scr)
     max = wwin;
     d = 0xffffffff;
     while (tmp) {
-	if ((tmp->flags.mapped || canFocusShaded(tmp))
-            && tmp->window_flags.focusable &&
-            !tmp->window_flags.skip_window_list) {
+	if (wWindowCanReceiveFocus(tmp) &&
+            (!tmp->window_flags.skip_window_list
+	     || tmp->flags.internal_window)) {
 	    if (max->client_win < tmp->client_win)
 	      max = tmp;
 	    if (tmp->client_win < wwin->client_win
@@ -254,13 +257,44 @@ PrevFocusWindow(WScreen *scr)
     return closest;
 }
 
-#if 1
+
+
+#if 0
+/*
+ * Is win2 below win1?
+ */
+static Bool
+isBelow(WWindow *win1, WWindow *win2)
+{
+    int i;
+    WCoreWindow *tmp;
+    
+    tmp = win1->frame->core->stacking->under;
+    while (tmp) {
+	if (tmp == win2->frame->core)
+	    return True;
+	tmp = tmp->stacking->under;
+    }
+    
+    for (i=win1->frame->window_level-1; i>=0; i--) {
+	tmp = win1->screen_ptr->stacking_list[i];
+	while (tmp) {
+	    if (tmp == win2->frame->core)
+		return True;
+	    tmp = tmp->stacking->under;
+	}
+    }
+    return True;
+}
+#endif
+
+
+
 /*
  *  XFetchName Wrapper
  *
  */
-
-Status MyXFetchName(dpy, win, winname)
+Bool wFetchName(dpy, win, winname)
 Display *dpy;
 Window win;
 char **winname;
@@ -269,24 +303,34 @@ char **winname;
     char **list;
     int num;
 
-	if (XGetWMName(dpy, win, &text_prop) != 0 && text_prop.value
-	    && text_prop.nitems > 0) {
-		if (text_prop.encoding == XA_STRING)
-			*winname = (char *)text_prop.value;
-		else {
-			text_prop.nitems = strlen((char *)text_prop.value);
-			if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) >=
-				Success && num > 0 && *list) {
-				XFree(text_prop.value);
-				*winname = strdup(*list);
-				XFreeStringList(list);
-			} else
-				*winname = (char *)text_prop.value;
+    if (XGetWMName(dpy, win, &text_prop)) {
+	if (text_prop.value && text_prop.nitems > 0) {
+	    if (text_prop.encoding == XA_STRING) {
+		*winname = wstrdup((char *)text_prop.value);
+		XFree(text_prop.value);
+	    } else {
+		text_prop.nitems = strlen((char *)text_prop.value);
+		if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) >=
+		    Success && num > 0 && *list) {
+		    XFree(text_prop.value);
+		    *winname = wstrdup(*list);
+		    XFreeStringList(list);
+		} else {
+		    *winname = wstrdup((char *)text_prop.value);
+		    XFree(text_prop.value);
 		}
-		return 1;
-    }
+	    }
+	} else {
+	    /* the title is set, but it was set to none */
+	    *winname = wstrdup("");
+	}
+	return True;
+    } else {
+	/* the hint is probably not set */
 	*winname = NULL;
-    return 0;  
+	
+	return False;
+    }
 }
 
 /*
@@ -294,7 +338,7 @@ char **winname;
  *
  */
 
-Status MyXGetIconName(dpy, win, iconname)
+Bool wGetIconName(dpy, win, iconname)
 Display *dpy;
 Window win;
 char **iconname;
@@ -303,67 +347,50 @@ char **iconname;
     char **list;
     int num;
     
-	if (XGetWMIconName(dpy, win, &text_prop) != 0 && text_prop.value
-	    && text_prop.nitems > 0) {
-		if (text_prop.encoding == XA_STRING)
-			*iconname = (char *)text_prop.value;
-		else {
-			text_prop.nitems = strlen((char *)text_prop.value);
-			if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) >=
-				Success && num > 0 && *list) {
-				XFree(text_prop.value);
-				*iconname = strdup(*list);
-				XFreeStringList(list);
-			} else
-				*iconname = (char *)text_prop.value;
-		}
-		return 1;
-    }
-	*iconname = NULL;
-    return 0;
-}
-    
-    
-#if 0
-    Atom actual_type;
-    int actual_format;
-    unsigned long nitems, bytesafter;
-    char **list;
-    int num;
-    XTextProperty text_prop;
-    Status status_ret;
-
-    if (XGetWindowProperty(dpy, win, XA_WM_ICON_NAME, 0L, 200L, False,
-			   AnyPropertyType, &actual_type, &actual_format, 
-			   &nitems, &bytesafter, (unsigned char **) iconname)
-	== Success && actual_type != None) {
-	text_prop.value = (unsigned char*)*iconname;
-	text_prop.encoding = actual_type;
-	text_prop.format = actual_format;
-	text_prop.nitems = nitems;
-	if (text_prop.value && text_prop.value[0] == '\0') {
-	    *iconname = (char*)text_prop.value;
-	    return 1;
-	}
-	status_ret = XmbTextPropertyToTextList(dpy, &text_prop, &list, &num);
-	if (status_ret == Success && num > 0 && *list) {
-	    *iconname = *list;
-	    return 1;
-	} else if (status_ret > Success) {
+    if (XGetWMIconName(dpy, win, &text_prop) != 0 && text_prop.value
+	&& text_prop.nitems > 0) {
+	if (text_prop.encoding == XA_STRING)
 	    *iconname = (char *)text_prop.value;
-	    return 1;
-	} else 
-	  *iconname = NULL;
-    } else
-      *iconname = NULL;
-    return 0;
+	else {
+	    text_prop.nitems = strlen((char *)text_prop.value);
+	    if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) >=
+		Success && num > 0 && *list) {
+		XFree(text_prop.value);
+		*iconname = wstrdup(*list);
+		XFreeStringList(list);
+	    } else
+		*iconname = (char *)text_prop.value;
+	}
+	return True;
+    }
+    *iconname = NULL;
+    return False;
+}
+
+
+#if 0
+#ifdef I18N_MB
+void
+wTextWidth(XFontSet font, char *text, int length) 
+{
+    XRectangle rect;
+    XRectangle AIXsucks;
+    
+    XmbTextExtents(font, text, length, &AIXsucks, &rec);
+
+    return rect.width;
+}
+#else
+void
+wTextWidth(XFontStruct *font, char *text, int length) 
+{
+    return XTextWidth(font, text, length);
 }
 #endif
+#endif
 
-#endif /* I18N_MB */
-
-
-static void eatExpose()
+static void 
+eatExpose()
 {
     XEvent event, foo;
     
@@ -381,10 +408,27 @@ static void eatExpose()
     }
 }
 
+
 void
 SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 {
-    float dx, dy, x=from_x, y=from_y, sx, sy;
+    time_t time0 = time(NULL);
+    float dx, dy, x=from_x, y=from_y, sx, sy, px, py;
+    int dx_is_bigger=0;
+
+    /* animation parameters */
+    static struct {
+	int delay;
+	int steps;
+	int slowdown;
+    } apars[5] = {
+	{ICON_SLIDE_DELAY_UF, ICON_SLIDE_STEPS_UF, ICON_SLIDE_SLOWDOWN_UF},
+	{ICON_SLIDE_DELAY_F, ICON_SLIDE_STEPS_F, ICON_SLIDE_SLOWDOWN_F},
+	{ICON_SLIDE_DELAY_M, ICON_SLIDE_STEPS_M, ICON_SLIDE_SLOWDOWN_M},
+	{ICON_SLIDE_DELAY_S, ICON_SLIDE_STEPS_S, ICON_SLIDE_SLOWDOWN_S},
+	{ICON_SLIDE_DELAY_U, ICON_SLIDE_STEPS_U, ICON_SLIDE_SLOWDOWN_U}};
+    
+    
 
     dx = (float)(to_x-from_x);
     dy = (float)(to_y-from_y);
@@ -392,35 +436,56 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
     sy = (dy == 0 ? 0 : fabs(dy)/dy);
 
     if (fabs(dx) > fabs(dy)) {
-	if (dx!=0) {
-	    dy = (dy*ICON_SLIDE_ANIMATION_STEP)/dx;
-	    dx = ICON_SLIDE_ANIMATION_STEP;
-	}
-    } else {
-	if (dy!=0) {
-	    dx = (dx*ICON_SLIDE_ANIMATION_STEP)/dy;
-	    dy = ICON_SLIDE_ANIMATION_STEP;
-	}
+        dx_is_bigger = 1;
     }
-    dx = fabs(dx) * sx;
-    dy = fabs(dy) * sy;
 
-    if (dx == 0.0 && to_x != from_x) x = to_x;
-    if (dy == 0.0 && to_y != from_y) y = to_y;
-    while ((int)x != to_x || (int)y != to_y) {
-	x += dx;
-	y += dy;
-	if ((dx<0 && (int)x < to_x)
-	    || (dx>0 && (int)x > to_x))
-	  x = (float)to_x;
-	if ((dy<0 && (int)y < to_y)
-	    || (dy>0 && (int)y > to_y))
-	  y = (float)to_y;
-	XMoveWindow(dpy, win, (int)x, (int)y);
-	XFlush(dpy);
-#if (ICON_SLIDE_ANIMATION_DELAY>0)
-	wmsleep(ICON_SLIDE_ANIMATION_DELAY);
-#endif
+    if (dx_is_bigger) {
+        px = dx / apars[(int)wPreferences.icon_slide_speed].slowdown;
+        if (px < apars[(int)wPreferences.icon_slide_speed].steps && px > 0)
+            px = apars[(int)wPreferences.icon_slide_speed].steps;
+        else if (px > -apars[(int)wPreferences.icon_slide_speed].steps && px < 0)
+            px = -apars[(int)wPreferences.icon_slide_speed].steps;
+        py = (sx == 0 ? 0 : px*dy/dx);
+    } else {
+        py = dy / apars[(int)wPreferences.icon_slide_speed].slowdown;
+        if (py < apars[(int)wPreferences.icon_slide_speed].steps && py > 0)
+            py = apars[(int)wPreferences.icon_slide_speed].steps;
+        else if (py > -apars[(int)wPreferences.icon_slide_speed].steps && py < 0)
+            py = -apars[(int)wPreferences.icon_slide_speed].steps;
+        px = (sy == 0 ? 0 : py*dx/dy);
+    }
+
+    while (x != to_x || y != to_y) {
+	x += px;
+        y += py;
+        if ((px<0 && (int)x < to_x) || (px>0 && (int)x > to_x))
+            x = (float)to_x;
+        if ((py<0 && (int)y < to_y) || (py>0 && (int)y > to_y))
+            y = (float)to_y;
+
+        if (dx_is_bigger) {
+            px = px * (1.0 - 1/(float)apars[(int)wPreferences.icon_slide_speed].slowdown);
+            if (px < apars[(int)wPreferences.icon_slide_speed].steps && px > 0)
+                px = apars[(int)wPreferences.icon_slide_speed].steps;
+            else if (px > -apars[(int)wPreferences.icon_slide_speed].steps && px < 0)
+                px = -apars[(int)wPreferences.icon_slide_speed].steps;
+            py = (sx == 0 ? 0 : px*dy/dx);
+        } else {
+            py = py * (1.0 - 1/(float)apars[(int)wPreferences.icon_slide_speed].slowdown);
+            if (py < apars[(int)wPreferences.icon_slide_speed].steps && py > 0)
+                py = apars[(int)wPreferences.icon_slide_speed].steps;
+            else if (py > -apars[(int)wPreferences.icon_slide_speed].steps && py < 0)
+                py = -apars[(int)wPreferences.icon_slide_speed].steps;
+            px = (sy == 0 ? 0 : py*dx/dy);
+        }
+
+        XMoveWindow(dpy, win, (int)x, (int)y);
+        XFlush(dpy);
+        if (apars[(int)wPreferences.icon_slide_speed].delay > 0) {
+            wusleep(apars[(int)wPreferences.icon_slide_speed].delay*1000L);
+        }
+	if (time(NULL) - time0 > MAX_ANIMATION_TIME)
+	    break;
     }
     XMoveWindow(dpy, win, to_x, to_y);
 
@@ -433,17 +498,19 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 char*
 ShrinkString(WFont *font, char *string, int width)
 {
+#ifndef I18N_MB
     int w, w1=0;
     int p;
     char *pos;
     char *text;
     int p1, p2, t;
+#endif
 
 #ifdef I18N_MB
-    return strdup(string);
+    return wstrdup(string);
 #else
     p = strlen(string);
-    w = MyTextWidth(font->font, string, p);
+    w = wTextWidth(font->font, string, p);
     text = wmalloc(strlen(string)+8);
     strcpy(text, string);
     if (w<=width)
@@ -456,7 +523,7 @@ ShrinkString(WFont *font, char *string, int width)
     if (pos) {
 	*pos = 0;
 	p = strlen(text);
-	w1=MyTextWidth(font->font, text, p);
+	w1=wTextWidth(font->font, text, p);
 	if (w1>width) {
 	    w1 = 0;
 	    p = 0;
@@ -473,14 +540,14 @@ ShrinkString(WFont *font, char *string, int width)
 	*text=0;
     }
     strcat(text, "...");
-    width -= MyTextWidth(font->font, "...", 3);
+    width -= wTextWidth(font->font, "...", 3);
     
     pos = string;
     p1=0;
     p2=p;
     t = (p2-p1)/2;
     while (p2>p1 && p1!=t) {
-	w = MyTextWidth(font->font, &string[p-t], t);
+	w = wTextWidth(font->font, &string[p-t], t);
 	if (w>width) {
 	    p2 = t;
 	    t = p1+(p2-p1)/2;
@@ -504,51 +571,16 @@ FindImage(char **paths, char *file)
     tmp = strrchr(file, ':');
     if (tmp) {
 	*tmp = 0;
-	path = find_file_in_list(paths, file);
+	path = wfindfileinlist(paths, file);
 	*tmp = ':';
     }
     if (!tmp || !path) {
-	path = find_file_in_list(paths, file);
+	path = wfindfileinlist(paths, file);
     }
     return path;
 }
 
 
-
-RImage*
-wGetImageForWindowName(WScreen *scr, char *wclass, char *winstance)
-{
-    char *file_name;
-    char *path;
-    int w, h;
-    RImage *image, *tmp;
-
-    file_name = wDefaultGetIconFile(scr, winstance, wclass);
-    if (!file_name)
-	return NULL;
-    
-    path = FindImage(wPreferences.pixmap_path, file_name);
-    
-    if (!path) {
-	wWarning(_("could not find icon file \"%s\""), file_name);
-	return NULL;
-    }
-    
-    image = RLoadImage(scr->rcontext, path, 0);
-    free(path);
-    
-    if (wPreferences.icon_size!=ICON_WIDTH && image) {
-	w = image->width*wPreferences.icon_size/ICON_WIDTH;
-	h = image->height*wPreferences.icon_size/ICON_HEIGHT;
-	
-	tmp = RScaleImage(image, w, h);
-	RDestroyImage(image);
-	image = tmp;
-    }
-    return image;
-}
-
-
 char*
 FlattenStringList(char **list, int count)
 {
@@ -557,7 +589,7 @@ FlattenStringList(char **list, int count)
 
     j = 0;
     for (i=0; i<count; i++) {
-        if (list[i]!=NULL) {
+        if (list[i]!=NULL && list[i][0]!=0) {
             j += strlen(list[i]);
             if (strpbrk(list[i], " \t"))
                 j += 2;
@@ -569,9 +601,9 @@ FlattenStringList(char **list, int count)
 	return NULL;
     }
 
-    strcpy(flat_string, list[0]);
-    for (i=1; i<count; i++) {
-	if (list[i]!=NULL) {
+    *flat_string = 0;
+    for (i=0; i<count; i++) {
+	if (list[i]!=NULL && list[i][0]!=0) {
             strcat(flat_string, " ");
             wspace = strpbrk(list[i], " \t");
             if (wspace)
@@ -581,46 +613,166 @@ FlattenStringList(char **list, int count)
                 strcat(flat_string, "\"");
 	}
     }
-    
+
     return flat_string;
 }
 
 
 
+/*
+ *----------------------------------------------------------------------
+ * ParseCommand --
+ * 	Divides a command line into a argv/argc pair.
+ *---------------------------------------------------------------------- 
+ */
+#define PRC_ALPHA	0
+#define PRC_BLANK	1
+#define PRC_ESCAPE	2
+#define PRC_DQUOTE	3
+#define PRC_EOS		4
+#define PRC_SQUOTE	5
+
+typedef struct {
+    short nstate;
+    short output;
+} DFA;
+
+
+static DFA mtable[9][6] = {
+    {{3,1},{0,0},{4,0},{1,0},{8,0},{6,0}},
+    {{1,1},{1,1},{2,0},{3,0},{5,0},{1,1}},
+    {{1,1},{1,1},{1,1},{1,1},{5,0},{1,1}},
+    {{3,1},{5,0},{4,0},{1,0},{5,0},{6,0}},
+    {{3,1},{3,1},{3,1},{3,1},{5,0},{3,1}},
+    {{-1,-1},{0,0},{0,0},{0,0},{0,0},{0,0}}, /* final state */
+    {{6,1},{6,1},{7,0},{6,1},{5,0},{3,0}},
+    {{6,1},{6,1},{6,1},{6,1},{5,0},{6,1}},
+    {{-1,-1},{0,0},{0,0},{0,0},{0,0},{0,0}}, /* final state */
+};
+
+char*
+next_token(char *word, char **next)
+{
+    char *ptr;
+    char *ret, *t;
+    int state, ctype;
+
+    t = ret = wmalloc(strlen(word)+1);
+    ptr = word;
+    
+    state = 0;
+    *t = 0;
+    while (1) {
+	if (*ptr==0) 
+	    ctype = PRC_EOS;
+	else if (*ptr=='\\')
+	    ctype = PRC_ESCAPE;
+	else if (*ptr=='"')
+	    ctype = PRC_DQUOTE;
+	else if (*ptr=='\'')
+	    ctype = PRC_SQUOTE;
+	else if (*ptr==' ' || *ptr=='\t')
+	    ctype = PRC_BLANK;
+	else
+	    ctype = PRC_ALPHA;
+
+	if (mtable[state][ctype].output) {
+	    *t = *ptr; t++;
+	    *t = 0;
+	}
+	state = mtable[state][ctype].nstate;
+	ptr++;
+	if (mtable[state][0].output<0) {
+	    break;
+	}
+    }
+
+    if (*ret==0)
+	t = NULL;
+    else
+	t = wstrdup(ret);
+
+    free(ret);
+    
+    if (ctype==PRC_EOS)
+	*next = NULL;
+    else
+	*next = ptr;
+    
+    return t;
+}
+
+
+void
+ParseCommand(char *command, char ***argv, int *argc)
+{
+    LinkedList *list = NULL;
+    char *token, *line;
+    int count, i;
+
+    line = command;
+    do {
+	token = next_token(line, &line);
+	if (token) {	    
+	    list = list_cons(token, list);
+	}
+    } while (token!=NULL && line!=NULL);
+
+    count = list_length(list);
+    *argv = wmalloc(sizeof(char*)*count);
+    i = count;
+    while (list!=NULL) {
+	(*argv)[--i] = list->head;
+	list_remove_head(&list);
+    }
+    *argc = count;
+}
+
+#if 0
 static void
 timeup(void *foo)
 {
     *(int*)foo=1;
 }
-
+#endif
 static char*
 getselection(WScreen *scr)
 {
+    char *tmp;
+    extern char *W_GetTextSelection(); /* in WINGs */
+    
+    tmp = W_GetTextSelection(scr->wmscreen, XA_PRIMARY);
+    if (!tmp)
+	tmp = W_GetTextSelection(scr->wmscreen, XA_CUT_BUFFER0);
+    return tmp;
+
+#if 0
     XEvent event;
     int timeover=0;
-    WMagicNumber *id;
+    WMHandlerID *id;
     
 #ifdef DEBUG
     puts("getting selection");
 #endif
-    wRequestSelection(dpy, scr->no_focus_win, LastTimestamp);
+    RequestSelection(dpy, scr->no_focus_win, LastTimestamp);
     /* timeout on 1 sec. */
-    id = wAddTimerHandler(1000, timeup, &timeover);
+    id = WMAddTimerHandler(1000, timeup, &timeover);
     while (!timeover) {
-	wNextEvent(&event);
+	WMNextEvent(dpy, &event);
 	if (event.type == SelectionNotify 
 	    && event.xany.window==scr->no_focus_win) {
-	    wDeleteTimerHandler(id);
+	    WMDeleteTimerHandler(id);
 #ifdef DEBUG
 	    puts("selection ok");
 #endif
-	    return wGetSelection(dpy, scr->no_focus_win);
+	    return GetSelection(dpy, scr->no_focus_win);
 	} else {
-	    DispatchEvent(&event);
+	    WMHandleEvent(&event);
 	}
     }
-    wWarning(_("selection timed-out"));
+    wwarning(_("selection timed-out"));
     return NULL;
+#endif
 }
 
 
@@ -652,7 +804,7 @@ getuserinput(WScreen *scr, char *line, int *ptr)
     }
 
     ret = NULL;
-    if (wInputDialog(scr, tmp, _("Enter command arguments:"), &ret)!= WDB_OK)
+    if (!wInputDialog(scr, tmp, _("Enter command arguments:"), &ret))
 	return NULL;
     else
 	return ret;
@@ -669,26 +821,30 @@ get_dnd_selection(WScreen *scr)
     char *flat_string;
     int count;
     
+#ifdef XDE_DND
+    if(scr->xdestring) {
+	return (wstrdup(scr->xdestring));
+    }
+#endif
     result=XGetTextProperty(dpy, scr->root_win, &text_ret, _XA_DND_SELECTION);
     
     if (result==0 || text_ret.value==NULL || text_ret.encoding==None
 	|| text_ret.format==0 || text_ret.nitems == 0) {
-	wWarning(_("unable to get dropped data from DND drop"));
+	wwarning(_("unable to get dropped data from DND drop"));
 	return NULL;
     }
     
-
     XTextPropertyToStringList(&text_ret, &list, &count);
     
     if (!list || count<1) {
 	XFree(text_ret.value);
-	wWarning(_("error getting dropped data from DND drop"));
+	wwarning(_("error getting dropped data from DND drop"));
 	return NULL;
     }
 
     flat_string = FlattenStringList(list, count);
     if (!flat_string) {
-	wWarning(_("out of memory while getting data from DND drop"));
+	wwarning(_("out of memory while getting data from DND drop"));
     }
     
     XFreeStringList(list);
@@ -732,7 +888,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
     olen = len+1;
     out = malloc(olen);
     if (!out) {
-	wWarning(_("out of memory during expansion of \"%s\""));
+	wwarning(_("out of memory during expansion of \"%s\""));
 	return NULL;
     }
     *out = 0;
@@ -786,7 +942,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		    olen += slen;
 		    nout = realloc(out,olen);
 		    if (!nout) {
-			wWarning(_("out of memory during expansion of \"%w\""));
+			wwarning(_("out of memory during expansion of \"%w\""));
 			goto error;
 		    }
 		    out = nout;
@@ -805,12 +961,16 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		    olen += slen;
 		    nout = realloc(out,olen);
 		    if (!nout) {
-			wWarning(_("out of memory during expansion of \"%a\""));
+			wwarning(_("out of memory during expansion of \"%a\""));
 			goto error;
 		    }
 		    out = nout;
 		    strcat(out,user_input);
 		    optr+=slen;
+		} else {
+		    /* Not an error, but user has Canceled the dialog box.
+		     * This will make the command to not be performed. */
+		    goto error;
 		}
 		break;
 
@@ -827,7 +987,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		olen += slen;
 		nout = realloc(out,olen);
 		if (!nout) {
-		    wWarning(_("out of memory during expansion of \"%d\""));
+		    wwarning(_("out of memory during expansion of \"%d\""));
 		    goto error;
 		}
 		out = nout;
@@ -838,20 +998,17 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		
 	     case 's':
 		if (!selection) {
-		    if (!XGetSelectionOwner(dpy, XA_PRIMARY)) {
-			wWarning(_("selection not available"));
-			goto error;
-		    }
 		    selection = getselection(scr);
 		}
 		if (!selection) {
+		    wwarning(_("selection not available"));
 		    goto error;
 		}
 		slen = strlen(selection);
 		olen += slen;
 		nout = realloc(out,olen);
 		if (!nout) {
-		    wWarning(_("out of memory during expansion of \"%s\""));
+		    wwarning(_("out of memory during expansion of \"%s\""));
 		    goto error;
 		}
 		out = nout;
@@ -896,12 +1053,252 @@ StringCompareHook(proplist_t pl1, proplist_t pl2)
 }
 
 
+/* feof doesn't seem to work on pipes */
 int
-isEof(FILE * stream) {      /* feof doesn't seem to work on pipes */
-
+IsEof(FILE * stream)
+{
     static struct stat stinfo;
 
     fstat(fileno(stream), &stinfo);
     return ((S_ISFIFO(stinfo.st_dev) && stinfo.st_size == 0) || 
             feof(stream));
 }
+
+
+void
+ParseWindowName(proplist_t value, char **winstance, char **wclass, char *where)
+{
+    char *name;
+
+    *winstance = *wclass = NULL;
+
+    if (!PLIsString(value)) {
+	wwarning(_("bad window name value in %s state info"), where);
+	return;
+    }
+
+    name = PLGetString(value);
+    if (!name || strlen(name)==0) {
+	wwarning(_("bad window name value in %s state info"), where);
+	return;
+    }
+
+    UnescapeWM_CLASS(name, winstance, wclass);
+}
+
+
+#if 0
+static char*
+keysymToString(KeySym keysym, unsigned int state)
+{
+    XKeyEvent kev;
+    char *buf = wmalloc(20);
+    int count;
+
+    kev.display = dpy;
+    kev.type = KeyPress;
+    kev.send_event = False;
+    kev.window = DefaultRootWindow(dpy);
+    kev.root = DefaultRootWindow(dpy);
+    kev.same_screen = True;
+    kev.subwindow = kev.root;
+    kev.serial = 0x12344321;
+    kev.time = CurrentTime;
+    kev.state = state;
+    kev.keycode = XKeysymToKeycode(dpy, keysym);
+    count = XLookupString(&kev, buf, 19, NULL, NULL);
+    buf[count] = 0;
+
+    return buf;
+}
+#endif
+
+char*
+GetShortcutString(char *text)
+{
+    char *buffer = NULL;
+    char *k;
+    int modmask = 0;
+/*    KeySym ksym;*/
+    int control = 0;
+    char *tmp;
+    
+    tmp = text = wstrdup(text);
+
+    /* get modifiers */
+    while ((k = strchr(text, '+'))!=NULL) {
+	int mod;
+	
+	*k = 0;
+	mod = wXModifierFromKey(text);
+	if (mod<0) {
+	    return wstrdup("bug");
+	}
+		
+	modmask |= mod;
+	
+	if (strcasecmp(text, "Meta")==0) {
+	    buffer = wstrappend(buffer, "M+");
+	} else if (strcasecmp(text, "Alt")==0) {
+	    buffer = wstrappend(buffer, "A+");
+	} else if (strcasecmp(text, "Shift")==0) {
+	    buffer = wstrappend(buffer, "Sh+");
+	} else if (strcasecmp(text, "Mod1")==0) {
+	    buffer = wstrappend(buffer, "M1+"); 
+	} else if (strcasecmp(text, "Mod2")==0) {
+	    buffer = wstrappend(buffer, "M2+"); 
+	} else if (strcasecmp(text, "Mod3")==0) {
+	    buffer = wstrappend(buffer, "M3+");
+	} else if (strcasecmp(text, "Mod4")==0) {
+	    buffer = wstrappend(buffer, "M4+"); 
+	} else if (strcasecmp(text, "Mod5")==0) {
+	    buffer = wstrappend(buffer, "M5+"); 
+	} else if (strcasecmp(text, "Control")==0) {
+	    control = 1;
+	} else {
+	    buffer = wstrappend(buffer, text);
+	}
+	text = k+1;
+    }
+
+    if (control) {
+	buffer = wstrappend(buffer, "^");
+    }
+    buffer = wstrappend(buffer, text);
+    
+    /* get key */
+/*    ksym = XStringToKeysym(text);
+    tmp = keysymToString(ksym, modmask);
+    puts(tmp);
+    buffer = wstrappend(buffer, tmp);
+*/
+    free(tmp);
+
+    return buffer;
+}
+
+
+char*
+EscapeWM_CLASS(char *name, char *class)
+{
+    char *ret;
+    char *ename = NULL, *eclass = NULL;
+    int i, j, l;
+
+    if (!name && !class)
+	return NULL;
+
+    if (name) {
+	l = strlen(name);
+	ename = wmalloc(l*2);
+	j = 0;
+	for (i=0; i<l; i++) {
+	    if (name[i]=='\\') {
+		ename[j++] = '\\';
+	    } else if (name[i]=='.') {
+		ename[j++] = '\\';
+	    }
+	    ename[j++] = name[i];
+	}
+	ename[j] = 0;
+    }
+    if (class) {
+	l = strlen(class);
+	eclass = wmalloc(l*2);
+	j = 0;
+	for (i=0; i<l; i++) {
+	    if (class[i]=='\\') {
+		eclass[j++] = '\\';
+	    } else if (class[i]=='.') {
+		eclass[j++] = '\\';
+	    }
+	    eclass[j++] = class[i];
+	}
+	eclass[j] = 0;
+    }
+
+    if (ename && eclass) {
+	ret = wmalloc(strlen(ename)+strlen(eclass)+4);
+	sprintf(ret, "%s.%s", ename, eclass);
+	free(ename);
+	free(eclass);
+    } else if (ename) {
+	ret = wstrdup(ename);
+	free(ename);
+    } else {
+	ret = wstrdup(eclass);
+	free(eclass);
+    }
+
+    return ret;
+}
+
+
+void
+UnescapeWM_CLASS(char *str, char **name, char **class)
+{
+    int i, j, k, dot;
+    Bool esc;
+    
+    j = strlen(str);
+    *name = wmalloc(j);
+    **name = 0;
+    *class = wmalloc(j);
+    **class = 0;
+
+    /* separate string in 2 parts */
+    esc = False;
+    dot = 0;
+    for (i = 0; i < j; i++) {
+	if (!esc) {
+	    if (str[i]=='\\') {
+		esc = True;
+	    } else if (str[i]=='.') {
+		dot = i;
+		break;
+	    }
+	} else {
+	    esc = False;
+	}
+    }
+
+    /* unescape strings */
+    esc = False;
+    k = 0;
+    for (i = 0; i < dot; i++) {
+	if (!esc) {
+	    if (str[i]=='\\') {
+		esc = True;
+	    } else {
+		(*name)[k++] = str[i];
+	    }
+	} else {
+	    esc = False;
+	}
+    }
+    (*name)[k] = 0;
+
+    esc = False;
+    k = 0;
+    for (i = dot+1; i<j; i++) {
+	if (!esc) {
+	    if (str[i]=='\\') {
+		esc = True;
+	    } else {
+		(*class)[k++] = str[i];
+	    }
+	} else {
+	    esc = False;
+	}
+    }
+    (*class)[k] = 0;
+    
+    if (!*name) {
+	free(*name);
+	*name = NULL;
+    }
+    if (!*class) {
+	free(*class);
+	*class = NULL;
+    }
+}
