@@ -1,7 +1,7 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 
@@ -24,15 +25,28 @@
 
 #include "GNUstep.h"
 
+#ifdef MWM_HINTS
+# include "motif.h"
+#endif
+
+
 int PropGetNormalHints(Window window, XSizeHints *size_hints, int *pre_iccm);
 void PropGetProtocols(Window window, WProtocols *prots);
 int PropGetWMClass(Window window, char **wm_class, char **wm_instance);
 int PropGetGNUstepWMAttr(Window window, GNUstepWMAttributes **attr);
 void PropWriteGNUstepWMAttr(Window window, GNUstepWMAttributes *attr);
-int PropGetClientLeader(Window window, Window *leader);
 
 void PropSetWMakerProtocols(Window root);
 void PropCleanUp(Window root);
 
+#ifdef MWM_HINTS
+int PropGetMotifWMHints(Window window, MWMHints **mwmhints);
+#endif
+
+Window PropGetClientLeader(Window window);
+
+#ifdef R6SM
+char *PropGetClientID(Window window);
+#endif
 
 #endif
