@@ -1,8 +1,8 @@
 /* screen.c - screen management
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 #include "wconfig.h"
@@ -33,7 +34,6 @@
 
 #include <wraster.h>
 
-#include "generic/wwmlib.h"
 #include "WindowMaker.h"
 #include "def_pixmaps.h"
 #include "screen.h"
@@ -46,6 +46,8 @@
 #include "dock.h"
 #include "resources.h"
 #include "workspace.h"
+#include "session.h"
+#include "balloon.h"
 
 #include <proplist.h>
 
@@ -58,17 +60,45 @@
 		|KeyPressMask|KeyReleaseMask)
 
 
-#define STIPPLE_WIDTH 2
-#define STIPPLE_HEIGHT 2
-static char STIPPLE_DATA[] = {0x02, 0x01};
-
-
 /**** Global variables ****/
 
 extern Cursor wCursor[WCUR_LAST];
 extern WPreferences wPreferences;
 extern Atom _XA_WINDOWMAKER_STATE;
 
+
+extern int wScreenCount;
+
+extern WDDomain *WDWindowMaker;
+
+
+/**** Local ****/
+
+#define STIPPLE_WIDTH 2
+#define STIPPLE_HEIGHT 2
+static char STIPPLE_DATA[] = {0x02, 0x01};
+
+static int CantManageScreen = 0;
+
+static proplist_t dApplications = NULL;
+static proplist_t dWorkspace;
+static proplist_t dDock;
+static proplist_t dClip;
+
+
+static void
+make_keys()
+{
+    if (dApplications!=NULL)
+        return;
+
+    dApplications = PLMakeString("Applications");
+    dWorkspace = PLMakeString("Workspace");
+    dDock = PLMakeString("Dock");
+    dClip = PLMakeString("Clip");
+}
+
+
 /*
  *----------------------------------------------------------------------
  * alreadyRunningError--
@@ -80,14 +110,13 @@ extern Atom _XA_WINDOWMAKER_STATE;
  * 	Nothing, unless something really evil happens...
  * 
  * Side effects:
- * 	The program is aborted.
+ * 	CantManageScreen is set to 1;
  *---------------------------------------------------------------------- 
  */
 static int 
 alreadyRunningError(Display *dpy, XErrorEvent *error)
 {
-    wFatal(_("it seems that there already is a window manager running"));
-    exit(1);
+    CantManageScreen = 1;
     return -1;
 }
 
@@ -127,10 +156,14 @@ allocButtonPixmaps(WScreen *scr)
     if (pix)
       pix->shared = 1;
     scr->b_pixmaps[WBUT_ICONIFY] = pix;
+    
+    pix = wPixmapCreateFromXPMData(scr, PRED_KILL_XPM);
+    if (pix)
+      pix->shared = 1;
+    scr->b_pixmaps[WBUT_KILL] = pix;
 }
 
 
-#ifdef DOCK
 
 static void
 draw_dot(WScreen *scr, Drawable d, int x, int y, GC gc)
@@ -143,6 +176,7 @@ draw_dot(WScreen *scr, Drawable d, int x, int y, GC gc)
     XDrawPoint(dpy, d, gc, x+1, y+1);
 }
 
+
 static WPixmap*
 make3Dots(WScreen *scr)
 {
@@ -165,7 +199,8 @@ make3Dots(WScreen *scr)
     mask = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size, 
 			 wPreferences.icon_size, 1);
     gcv.foreground = 0;
-    gc2 = XCreateGC(dpy, mask, GCForeground, &gcv);
+    gcv.graphics_exposures = False;
+    gc2 = XCreateGC(dpy, mask, GCForeground|GCGraphicsExposures, &gcv);
     XFillRectangle(dpy, mask, gc2, 0, 0, wPreferences.icon_size, 
 		   wPreferences.icon_size);
     XSetForeground(dpy, gc2, 1);
@@ -181,71 +216,6 @@ make3Dots(WScreen *scr)
     return wpix;
 }
 
-#if 0
-static WPixmap*
-makeTBolt(WScreen *scr)
-{
-    WPixmap *pix;
-    GC gc2, gc;
-    XGCValues gcv;
-    Pixmap p, m;
-#if 0
-    pix = wmalloc(sizeof(WPixmap));
-    memset(pix, 0, sizeof(WPixmap));
-    pix->shared = 1;
-    pix->width = wPreferences.icon_size;
-    pix->height = wPreferences.icon_size;
-    pix->depth = scr->w_depth;
-    gc = scr->copy_gc;
-    pix->image = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size, 
-			       wPreferences.icon_size, scr->w_depth);
-    XSetForeground(dpy, gc, scr->black_pixel);
-
-    pix->mask = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size, 
-			      wPreferences.icon_size, 1);
-    gcv.foreground = 0;
-    gc2 = XCreateGC(dpy, pix->mask, GCForeground, &gcv);
-    XFillRectangle(dpy, pix->mask, gc2, 0, 0, wPreferences.icon_size, 
-		   wPreferences.icon_size);
-    XSetForeground(dpy, gc2, 1);
-    XDrawLine(dpy, pix->mask, gc2, wPreferences.icon_size - 4, 4, 
-	      wPreferences.icon_size - 7, 7);
-    XDrawLine(dpy, pix->mask, gc2, wPreferences.icon_size - 5, 7, 
-	      wPreferences.icon_size - 8, 11);
-    XDrawPoint(dpy, pix->mask, gc2, wPreferences.icon_size - 6, 7);
-    XFreeGC(dpy, gc2);
-#endif 
-    gc = scr->copy_gc;
-    p = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size, 
-		      wPreferences.icon_size, scr->w_depth);
-    XSetForeground(dpy, gc, scr->black_pixel);
-    XFillRectangle(dpy, p, gc, 0, 0, wPreferences.icon_size, 
-		   wPreferences.icon_size);
-    XSetForeground(dpy, gc, scr->white_pixel);
-    XDrawLine(dpy, p, gc, wPreferences.icon_size - 4, 5, 
-	      wPreferences.icon_size - 3, 5);
-    XDrawPoint(dpy, p, gc, wPreferences.icon_size - 3, 4);
-
-    m = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size, 
-		      wPreferences.icon_size, 1);
-    gcv.foreground = 0;
-    gc2 = XCreateGC(dpy, m, GCForeground, &gcv);
-    XFillRectangle(dpy, m, gc2, 0, 0, wPreferences.icon_size, 
-		   wPreferences.icon_size);
-    XSetForeground(dpy, gc2, 1);
-    XFillRectangle(dpy, m, gc2, wPreferences.icon_size - 5, 4, 3, 2);
-    
-    XFreeGC(dpy, gc2);
-
-    pix = wPixmapCreate(scr, p, m);
-    pix->shared = 1;
-
-    return pix;
-}
-#endif
-#endif /* DOCK */
-
-
 
 static void
 allocGCs(WScreen *scr)
@@ -262,14 +232,35 @@ allocGCs(WScreen *scr)
     gcv.stipple = scr->stipple_bitmap;
     gcv.foreground = scr->white_pixel;
     gcv.fill_style = FillStippled;
-    gcm = GCForeground|GCStipple|GCFillStyle;
+    gcv.graphics_exposures = False;
+    gcm = GCForeground|GCStipple|GCFillStyle|GCGraphicsExposures;
     scr->stipple_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
 
-    
-    gcm = GCForeground;
+   
+    /* selected icon border GCs */
+    gcv.function    = GXcopy;
+    gcv.foreground  = scr->white_pixel;
+    gcv.background  = scr->black_pixel;
+    gcv.line_width  = 1;
+    gcv.line_style  = LineDoubleDash;
+    gcv.fill_style  = FillSolid;
+    gcv.dash_offset = 0;
+    gcv.dashes      = 4;
+    gcv.graphics_exposures = False;
+   
+    gcm = GCFunction | GCGraphicsExposures;
+    gcm |= GCForeground | GCBackground;
+    gcm |= GCLineWidth | GCLineStyle;
+    gcm |= GCFillStyle;
+    gcm |= GCDashOffset | GCDashList;
+
+    scr->icon_select_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
+
+    gcm = GCForeground|GCGraphicsExposures;
     
     scr->menu_title_pixel[0] = scr->white_pixel;
     gcv.foreground = scr->white_pixel;
+    gcv.graphics_exposures = False;
     scr->menu_title_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
 
     scr->mtext_pixel = scr->black_pixel;
@@ -277,66 +268,88 @@ allocGCs(WScreen *scr)
     scr->menu_entry_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
 
     /* selected menu entry GC */
-    gcm = GCForeground|GCBackground;
+    gcm = GCForeground|GCBackground|GCGraphicsExposures;
     gcv.foreground = scr->white_pixel;
     gcv.background = scr->white_pixel;
+    gcv.graphics_exposures = False;
     scr->select_menu_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
 
-    
+    /* disabled menu entry GC */
     scr->dtext_pixel = scr->black_pixel;
-    gcm = GCForeground|GCBackground|GCStipple;
-    gcv.foreground = color.pixel;
+    gcm = GCForeground|GCBackground|GCStipple|GCGraphicsExposures;
     gcv.stipple = scr->stipple_bitmap;
-
+    gcv.graphics_exposures = False;
     scr->disabled_menu_entry_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
 
     /* frame GC */
     wGetColor(scr, DEF_FRAME_COLOR, &color);
-
-    gcv.foreground = color.pixel;
     gcv.function = GXxor;
+    /* this will raise the probability of the XORed color being different
+     * of the original color in PseudoColor when not all color cells are
+     * initialized */
+    if (DefaultVisual(dpy, scr->screen)->class==PseudoColor)
+    	gcv.plane_mask = (1<<(scr->depth-1))|1;
+    else
+	gcv.plane_mask = AllPlanes;
+    gcv.foreground = color.pixel;
+    if (gcv.foreground == 0)
+	gcv.foreground = 1;
     gcv.line_width = DEF_FRAME_THICKNESS;
     gcv.subwindow_mode = IncludeInferiors;
-    scr->frame_gc = XCreateGC(dpy, scr->root_win, GCForeground
-			      |GCFunction|GCSubwindowMode|GCLineWidth, &gcv);
+    gcv.graphics_exposures = False;
+    scr->frame_gc = XCreateGC(dpy, scr->root_win, GCForeground|GCGraphicsExposures
+			      |GCFunction|GCSubwindowMode|GCLineWidth
+			      |GCPlaneMask, &gcv);
     
     /* line GC */
     gcv.foreground = color.pixel;
+
+    if (gcv.foreground == 0)
+        /* XOR:ing with a zero is not going to be of much use, so
+           in that case, we somewhat arbitrarily xor with 17 instead. */
+        gcv.foreground = 17;
+
     gcv.function = GXxor;
     gcv.subwindow_mode = IncludeInferiors;
     gcv.line_width = 1;
     gcv.cap_style = CapRound;
-    gcm = GCForeground|GCFunction|GCSubwindowMode|GCLineWidth|GCCapStyle;
+    gcv.graphics_exposures = False;
+    gcm = GCForeground|GCFunction|GCSubwindowMode|GCLineWidth|GCCapStyle
+	|GCGraphicsExposures;
     scr->line_gc = XCreateGC(dpy, scr->root_win, gcm, &gcv);
 
-    scr->line_pixel = color.pixel;
+    scr->line_pixel = gcv.foreground;
     
     /* copy GC */
     gcv.foreground = scr->white_pixel;
     gcv.background = scr->black_pixel;
-    scr->copy_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground, &gcv);
-
-    /* fiend GC */
-    gcv.foreground = scr->white_pixel;
-    gcv.background = scr->black_pixel;
-    scr->fiend_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground, &gcv);
+    gcv.graphics_exposures = False;
+    scr->copy_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
+			     |GCGraphicsExposures, &gcv);
 
     /* window title text GC */
-    scr->window_title_gc = XCreateGC(dpy, scr->w_win, 0, NULL);
+    gcv.graphics_exposures = False;
+    scr->window_title_gc = XCreateGC(dpy, scr->w_win,GCGraphicsExposures,&gcv);
 
     /* icon title GC */
-    scr->icon_title_gc = XCreateGC(dpy, scr->w_win, 0, NULL);
+    scr->icon_title_gc = XCreateGC(dpy, scr->w_win, GCGraphicsExposures, &gcv);
 
-    /* fiend title GC */
-    scr->fiend_title_gc = XCreateGC(dpy, scr->w_win, 0, NULL);
+    /* clip title GC */
+    scr->clip_title_gc = XCreateGC(dpy, scr->w_win, GCGraphicsExposures, &gcv);
 
     /* move/size display GC */
-    gcv.foreground = scr->black_pixel;
-    gcm = GCForeground;
-    scr->info_text_gc = XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    gcv.graphics_exposures = False;
+    gcm = GCGraphicsExposures;
+    scr->info_text_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
+    
+    /* misc drawing GC */
+    gcv.graphics_exposures = False;
+    gcm = GCGraphicsExposures;
+    scr->draw_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);    
 }
 
 
+
 static void
 createPixmaps(WScreen *scr)
 {
@@ -346,50 +359,74 @@ createPixmaps(WScreen *scr)
     Pixmap p, m;
 
     /* load pixmaps */
-    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_INDICATOR_XBM_DATA, 
-				   (char*)MENU_INDICATOR_XBM_DATA,
-				   MENU_INDICATOR_XBM_SIZE,
-				   MENU_INDICATOR_XBM_SIZE,
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_RADIO_INDICATOR_XBM_DATA,
+				   (char*)MENU_RADIO_INDICATOR_XBM_DATA,
+				   MENU_RADIO_INDICATOR_XBM_SIZE,
+				   MENU_RADIO_INDICATOR_XBM_SIZE,
 				   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
       pix->shared = 1;
-    scr->menu_indicator = pix;
+    scr->menu_radio_indicator = pix;
 
-    
-    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_INDICATOR2_XBM_DATA,
-				   (char*)MENU_INDICATOR2_XBM_DATA,
-				   MENU_INDICATOR2_XBM_SIZE,
-				   MENU_INDICATOR2_XBM_SIZE,
+
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_CHECK_INDICATOR_XBM_DATA,
+				   (char*)MENU_CHECK_INDICATOR_XBM_DATA,
+				   MENU_CHECK_INDICATOR_XBM_SIZE,
+				   MENU_CHECK_INDICATOR_XBM_SIZE,
 				   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
       pix->shared = 1;
-    scr->menu_indicator2 = pix;
+    scr->menu_check_indicator = pix;
+
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_MINI_INDICATOR_XBM_DATA,
+				   (char*)MENU_MINI_INDICATOR_XBM_DATA,
+				   MENU_MINI_INDICATOR_XBM_SIZE,
+				   MENU_MINI_INDICATOR_XBM_SIZE,
+				   scr->black_pixel, scr->white_pixel);
+    if (pix!=NULL)
+      pix->shared = 1;
+    scr->menu_mini_indicator = pix;
+
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_HIDE_INDICATOR_XBM_DATA,
+				   (char*)MENU_HIDE_INDICATOR_XBM_DATA,
+				   MENU_HIDE_INDICATOR_XBM_SIZE,
+				   MENU_HIDE_INDICATOR_XBM_SIZE,
+				   scr->black_pixel, scr->white_pixel);
+    if (pix!=NULL)
+      pix->shared = 1;
+    scr->menu_hide_indicator = pix;
+
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_SHADE_INDICATOR_XBM_DATA,
+                                  (char*)MENU_SHADE_INDICATOR_XBM_DATA,
+                                  MENU_SHADE_INDICATOR_XBM_SIZE,
+                                  MENU_SHADE_INDICATOR_XBM_SIZE,
+                                  scr->black_pixel, scr->white_pixel);
+    if (pix!=NULL)
+      pix->shared = 1;
+    scr->menu_shade_indicator = pix;
 
 
-    image = wGetImageForWindowName(scr, "WMPanel", "Logo");
+    image = wDefaultGetImage(scr, "Logo", "WMPanel");
 
     if (!image) {
-	wWarning(_("could not load logo image for panels"));
+	wwarning(_("could not load logo image for panels"));
     } else {
 	if (!RConvertImageMask(scr->rcontext, image, &p, &m, 128)) {
-	    wWarning(_("error making logo image for panel:%s"), RErrorString);
+	    wwarning(_("error making logo image for panel:%s"), RMessageForError(RErrorCode));
 	} else {
 	    wmpix = WMCreatePixmapFromXPixmaps(scr->wmscreen, p, m,
 					       image->width, image->height, 
 					       scr->depth);
-	    WMSetApplicationIcon(scr->wmscreen, wmpix); WMReleasePixmap(wmpix);
+	    WMSetApplicationIconImage(scr->wmscreen, wmpix); 
+	    WMReleasePixmap(wmpix);
 	}
 	RDestroyImage(image);
     }
-    
-#ifdef DOCK
-    if (!wPreferences.flags.nodock || !wPreferences.flags.nofiend) {
+
+    if (!wPreferences.flags.nodock || !wPreferences.flags.noclip) {
 	scr->dock_dots = make3Dots(scr);
-	/*
-	scr->dock_tbolt = makeTBolt(scr);
-	 */
     }
-#endif
+
     /* titlebar button pixmaps */
     allocButtonPixmaps(scr);
 }
@@ -414,26 +451,30 @@ static void
 createInternalWindows(WScreen *scr)
 {
     int vmask;
-    int bfw;
     XSetWindowAttributes attribs;
 
-#ifdef I18N_MB
-    bfw = XmbTextEscapement(scr->info_text_font->font, "-8888 x -8888", 13);
-#else
-    bfw = XTextWidth(scr->info_text_font->font, "-8888 x -8888", 13);
-#endif
-
     /* window for displaying geometry information during resizes and moves */
-    vmask = CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    vmask = CWBorderPixel|CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    attribs.border_pixel = 0;
     attribs.save_under = True;
     attribs.override_redirect = True;    
     attribs.cursor = wCursor[WCUR_DEFAULT];
     attribs.background_pixmap = None;
-    attribs.background_pixel = scr->window_title_texture[WS_UNFOCUSED]->any.color.pixel;
+    if (scr->resizebar_texture[0])
+	attribs.background_pixel = scr->resizebar_texture[0]->normal.pixel;
+    else
+	attribs.background_pixel = scr->light_pixel;
+    vmask |= CWColormap;
+    attribs.colormap = scr->w_colormap;
+
+    wGetGeometryWindowSize(scr, &scr->geometry_display_width,
+			   &scr->geometry_display_height);
     scr->geometry_display =
-      XCreateWindow(dpy, scr->root_win, (scr->scr_width-bfw)/2, 
-		    (scr->scr_height-20)/2, bfw, 20, 1, scr->depth, 
-		    CopyFromParent, scr->w_visual, vmask, &attribs);
+      XCreateWindow(dpy, scr->root_win, 1, 1,
+		    scr->geometry_display_width,
+		    scr->geometry_display_height,
+		    1, scr->w_depth, CopyFromParent, scr->w_visual,
+		    vmask, &attribs);
     
     /* InputOnly window to get the focus when no other window can get it */
     vmask = CWEventMask|CWOverrideRedirect;
@@ -446,18 +487,34 @@ createInternalWindows(WScreen *scr)
 
     XSetInputFocus (dpy, scr->no_focus_win, RevertToParent, CurrentTime);
  
-#ifdef DOCK
     /* shadow window for dock buttons */
-    vmask = CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    vmask = CWBorderPixel|CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    attribs.border_pixel = 0;
     attribs.save_under = True;
     attribs.override_redirect = True;
     attribs.background_pixmap = None;
     attribs.background_pixel = scr->white_pixel;
+    vmask |= CWColormap;
+    attribs.colormap = scr->w_colormap;
     scr->dock_shadow =
       XCreateWindow(dpy, scr->root_win, 0, 0, wPreferences.icon_size, 
-		    wPreferences.icon_size, 0, scr->depth, CopyFromParent, 
+		    wPreferences.icon_size, 0, scr->w_depth, CopyFromParent, 
 		    scr->w_visual, vmask, &attribs);
-#endif
+   
+    /* workspace name balloon for clip */
+    vmask = CWBackPixel|CWSaveUnder|CWOverrideRedirect|CWColormap;
+    attribs.save_under = True;
+    attribs.override_redirect = True;
+    attribs.colormap = scr->w_colormap;
+    attribs.background_pixel = scr->icon_back_texture->normal.pixel;
+    scr->clip_balloon =
+      XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth, 
+		    CopyFromParent, scr->w_visual, vmask, &attribs);
+    /*
+     * If the window is clicked without having ButtonPress selected, the
+     * resulting event will have event.xbutton.window == root.
+     */
+    XSelectInput(dpy, scr->clip_balloon, ButtonPressMask);
 }
 
 
@@ -479,38 +536,17 @@ createInternalWindows(WScreen *scr)
  * TODO: User specifiable visual.
  *---------------------------------------------------------------------- 
  */
-WScreen *
+WScreen*
 wScreenInit(int screen_number)
 {
     WScreen *scr;
     XIconSize icon_size[1];
     RContextAttributes rattr;
     extern int wVisualID;
-    WDDomain *domain;
-    
-    
-    domain = wDefaultsInitDomain("WindowMaker");
-    if (!domain->dictionary) {
-	wWarning(_("could not read domain \"%s\" from defaults database"),
-		 "WindowMaker");
-    }
-    
-    /* read defaults that don't change until a restart */
-    wReadStaticDefaults(domain ? domain->dictionary : NULL);
+    long event_mask;
+    WMColor *color;
+    XErrorHandler oldHandler;
 
-    /* swap mouse button functions */
-    if (wPreferences.swap_buttons) {
-	wLeftBtn = Button3;
-	wRightBtn = Button1;
-	wLeftBtnMask = Button3Mask;
-	wRightBtnMask = Button1Mask;
-    } else {
-	wLeftBtn = Button1;
-	wRightBtn = Button3;
-	wLeftBtnMask = Button1Mask;
-	wRightBtnMask = Button3Mask;
-    }
-    
     scr = wmalloc(sizeof(WScreen));
     memset(scr, 0, sizeof(WScreen));
 
@@ -523,10 +559,22 @@ wScreenInit(int screen_number)
     scr->scr_width = WidthOfScreen(ScreenOfDisplay(dpy, screen_number));
     scr->scr_height = HeightOfScreen(ScreenOfDisplay(dpy, screen_number));
 
-    XSetErrorHandler((XErrorHandler)alreadyRunningError);
-    XSelectInput(dpy, scr->root_win, EVENT_MASK);
-    XSync(dpy, 0);
-    XSetErrorHandler((XErrorHandler)NULL);
+    CantManageScreen = 0;
+    oldHandler = XSetErrorHandler((XErrorHandler)alreadyRunningError);
+
+    event_mask = EVENT_MASK;
+
+    if (wPreferences.disable_root_mouse)
+	event_mask &= ~(ButtonPressMask|ButtonReleaseMask);
+    XSelectInput(dpy, scr->root_win, event_mask);
+
+    XSync(dpy, False);
+    XSetErrorHandler(oldHandler);
+    
+    if (CantManageScreen) {
+	free(scr);
+	return NULL;
+    }
 
     XDefineCursor(dpy, scr->root_win, wCursor[WCUR_DEFAULT]);
 
@@ -543,62 +591,71 @@ wScreenInit(int screen_number)
     }
     scr->rcontext = RCreateContext(dpy, screen_number, &rattr);
     if (!scr->rcontext) {
-	wWarning(_("could not initialize graphics library context: %s"),
-		 RErrorString);
-	wAbort();
-	exit(1);
-    }
+	wwarning(_("could not initialize graphics library context: %s"),
+		 RMessageForError(RErrorCode));
+	wAbort(False);
+    } else {
+	char **formats;
+	int i = 0;
 
+	formats = RSupportedFileFormats();
+	if (formats) {
+	    for (i=0; formats[i]!=NULL; i++) {
+		if (strcmp(formats[i], "TIFF")==0) {
+		    scr->flags.supports_tiff = 1;
+		    break;
+		}
+	    }
+	    RFreeStringList(formats);
+	}
+    }
+    
     scr->w_win = scr->rcontext->drawable;
     scr->w_visual = scr->rcontext->visual;
     scr->w_depth = scr->rcontext->depth;
     scr->w_colormap = scr->rcontext->cmap;
 
     scr->black_pixel = scr->rcontext->black;
-    scr->white_pixel = scr->rcontext->white;
+    scr->white_pixel = scr->rcontext->white;    
+
+    /* create screen descriptor for WINGs */
+    scr->wmscreen = WMCreateScreenWithRContext(dpy, screen_number, 
+					       scr->rcontext);
+
+    color = WMGrayColor(scr->wmscreen);
+    scr->light_pixel = WMColorPixel(color);
+    WMReleaseColor(color);
+
+    color = WMDarkGrayColor(scr->wmscreen);
+    scr->dark_pixel = WMColorPixel(color);
+    WMReleaseColor(color);
+
+    {
+	XColor xcol;
+	/* frame boder color */
+	wGetColor(scr, FRAME_BORDER_COLOR, &xcol);
+	scr->frame_border_pixel = xcol.pixel;
+    }
 
-    
-    scr->wmscreen = WMCreateScreen(dpy, screen_number, scr->rcontext);
-    
-    
     /* create GCs with default values */
     allocGCs(scr);
-    
-    /* read defaults */
-    scr->dmain = domain;
-    wReadDefaults(scr, domain->dictionary);
 
-    /* init other domains */
-    scr->droot_menu = wDefaultsInitDomain("WMRootMenu");
-    if (!scr->droot_menu->dictionary) {
-	wWarning(_("could not read domain \"%s\" from defaults database"),
-		 "WMRootMenu");
-    }
-
-    scr->dwindow = wDefaultsInitDomain("WMWindowAttributes");
-    if (!scr->dwindow->dictionary) {
-	wWarning(_("could not read domain \"%s\" from defaults database"),
-		 "WMWindowAttributes");
-    }
+    /* read defaults for this screen */
+    wReadDefaults(scr, WDWindowMaker->dictionary);
 
     /* create initial workspace */
     wWorkspaceNew(scr);
 
-    
-    /* setup defaults file polling */
-    wAddTimerHandler(3000, wDefaultsCheckDomains, scr);
-    
-    
     createInternalWindows(scr);
 
     /* create shared pixmaps */
     createPixmaps(scr);
 
     /* set icon sizes we can accept from clients */
-    icon_size[0].min_width = 1;
-    icon_size[0].min_height = 1;
-    icon_size[0].max_width = wPreferences.icon_size;
-    icon_size[0].max_height = wPreferences.icon_size;
+    icon_size[0].min_width = 8;
+    icon_size[0].min_height = 8;
+    icon_size[0].max_width = wPreferences.icon_size-4;
+    icon_size[0].max_height = wPreferences.icon_size-4;
     icon_size[0].width_inc = 1;
     icon_size[0].height_inc = 1;
     XSetIconSizes(dpy, scr->root_win, icon_size, 1);
@@ -606,11 +663,15 @@ wScreenInit(int screen_number)
     /* setup WindowMaker protocols property in the root window*/
     PropSetWMakerProtocols(scr->root_win);
 
-    
+#ifdef BALLOON_TEXT
+    /* initialize balloon text stuff */
+    wBalloonInitialize(scr);
+#endif
+
     /* kluge to load menu configurations at startup */
     OpenRootMenu(scr, -10000, -10000, False);
     wMenuUnmap(scr->root_menu);
-    
+
     return scr;
 }
 
@@ -618,22 +679,39 @@ wScreenInit(int screen_number)
 void
 wScreenRestoreState(WScreen *scr)
 {
-#ifdef DOCK
-    proplist_t dDock, dock_state;
-#endif
+    proplist_t state;
+    char *path;
 
-    scr->session_state = PLGetProplistWithPath(wDefaultsPathForDomain("WMState"));
+    make_keys();
+
+    if (wScreenCount == 1)
+	path = wdefaultspathfordomain("WMState");
+    else {
+	char buf[16];
+	sprintf(buf, "WMState.%i", scr->screen);
+	path = wdefaultspathfordomain(buf);
+    }
+    scr->session_state = PLGetProplistWithPath(path);
+    free(path);
+    if (!scr->session_state && wScreenCount>1) {
+	char buf[16];
+	sprintf(buf, "WMState.%i", scr->screen);
+	path = wdefaultspathfordomain(buf);
+	scr->session_state = PLGetProplistWithPath(path);
+	free(path);
+    }
+
+    if (!wPreferences.flags.noclip) {
+        state = PLGetDictionaryEntry(scr->session_state, dClip);
+        scr->clip_icon = wClipRestoreState(scr, state);
+    }
 
     wWorkspaceRestoreState(scr);
-    
-#ifdef DOCK
+
     if (!wPreferences.flags.nodock) {
-        dDock = PLMakeString("Dock");
-        dock_state = PLGetDictionaryEntry(scr->session_state, dDock);
-        scr->dock = wDockRestoreState(scr, dock_state, WM_DOCK);
-        PLRelease(dDock);
+        state = PLGetDictionaryEntry(scr->session_state, dDock);
+        scr->dock = wDockRestoreState(scr, state, WM_DOCK);
     }
-#endif
 
 }
 
@@ -643,18 +721,24 @@ wScreenSaveState(WScreen *scr)
 {
     WWorkspaceState wstate;
     WWindow *wwin;
-    proplist_t path, old_state;
+    char *str;
+    proplist_t path, old_state, foo;
+    CARD32 data[2];
+
+
+    make_keys();
 
 /*
  * Save current workspace, so can go back to it upon restart.
  */
     wstate.workspace = scr->current_workspace;
 
+    data[0] = wstate.flags;
+    data[1] = wstate.workspace;
 
     XChangeProperty(dpy, scr->root_win, _XA_WINDOWMAKER_STATE,
                     _XA_WINDOWMAKER_STATE, 32, PropModeReplace,
-                    (unsigned char *) &wstate, 
-		    sizeof(WWorkspaceState)/sizeof(int));
+                    (unsigned char *) data, 2);
     
     /* save state of windows */
     wwin = scr->focused_window;
@@ -663,23 +747,65 @@ wScreenSaveState(WScreen *scr)
 	wwin = wwin->prev;
     }
 
+
+    if (wPreferences.flags.noupdates)
+	return;
+
+
     old_state = scr->session_state;
     scr->session_state = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
 
-#ifdef DOCK
+    PLSetStringCmpHook(NULL);
+
     /* save dock state to file */
-    if (!wPreferences.flags.nodock)
+    if (!wPreferences.flags.nodock) {
         wDockSaveState(scr);
-#endif
-    
-    wWorkspaceSaveState(scr);
+    } else {
+        if ((foo = PLGetDictionaryEntry(old_state, dDock))!=NULL) {
+            PLInsertDictionaryEntry(scr->session_state, dDock, foo);
+        }
+    }
+    if (!wPreferences.flags.noclip) {
+        wClipSaveState(scr);
+    } else {
+        if ((foo = PLGetDictionaryEntry(old_state, dClip))!=NULL) {
+            PLInsertDictionaryEntry(scr->session_state, dClip, foo);
+        }
+    }
+
+    wWorkspaceSaveState(scr, old_state);
+
+    if (wPreferences.save_session_on_exit) {
+        wSessionSaveState(scr);
+    } else {
+        if ((foo = PLGetDictionaryEntry(old_state, dApplications))!=NULL) {
+            PLInsertDictionaryEntry(scr->session_state, dApplications, foo);
+        }
+        if ((foo = PLGetDictionaryEntry(old_state, dWorkspace))!=NULL) {
+            PLInsertDictionaryEntry(scr->session_state, dWorkspace, foo);
+        }
+    }
+
+    /* clean up */
+    PLSetStringCmpHook(StringCompareHook);
 
     wMenuSaveState(scr);
 
-    path = PLMakeString(wDefaultsPathForDomain("WMState"));
+    if (wScreenCount == 1)
+	str = wdefaultspathfordomain("WMState");
+    else {
+	char buf[16];
+	sprintf(buf, "WMState.%i", scr->screen);
+	str = wdefaultspathfordomain(buf);
+    }
+    path = PLMakeString(str);
+    free(str);
     PLSetFilename(scr->session_state, path);
-    PLSave(scr->session_state, YES);
+    if (!PLSave(scr->session_state, YES)) {
+	wwarning(_("could not save session state in %s"), PLGetString(path));
+    }
     PLRelease(path);
+    PLRelease(old_state);
 }
 
 
@@ -689,12 +815,12 @@ wScreenBringInside(WScreen *scr, int *x, int *y, int width, int height)
 {
     int moved = 0;
     int tol_w, tol_h;
-    
+
     if (width > 20)
 	tol_w = width/2;
     else
 	tol_w = 20;
-    
+
     if (height > 20)
 	tol_h = height/2;
     else
@@ -706,7 +832,7 @@ wScreenBringInside(WScreen *scr, int *x, int *y, int width, int height)
 	*x = scr->scr_width - tol_w - 1, moved = 1;
 
     if (*y < -height + 10)
-	*y = -tol_w, moved = 1;
+	*y = -tol_h, moved = 1;
     else if (*y >= scr->scr_height - 10)
 	*y = scr->scr_height - tol_h - 1, moved = 1;
 
