@@ -1,7 +1,7 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 #include "wconfig.h"
@@ -26,7 +27,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
-#include "generic/wwmlib.h"
 #include "WindowMaker.h"
 #include "screen.h"
 #include "window.h"
@@ -41,25 +41,6 @@ extern XContext wStackContext;
 extern WPreferences wPreferences;
 
 
-static int
-levelToIndex(int level)
-{
-    switch (level) {
-     case NSNormalWindowLevel:
-	return 0;
-     case NSFloatingWindowLevel:
-	return 1;
-     case NSDockWindowLevel:
-	return 2;
-     case NSSubmenuWindowLevel:
-	return 3;
-     case NSMainMenuWindowLevel:
-	return 4;
-     default:
-	return 0;
-    }
-}
-
 /*
  *---------------------------------------------------------------------- 
  * RemakeStackList--
@@ -73,66 +54,6 @@ levelToIndex(int level)
  * 
  *---------------------------------------------------------------------- 
  */
-#ifdef BUGGYSTACKING
-void
-RemakeStackList(WScreen *scr)
-{
-    Window *windows;
-    unsigned int nwindows;
-    Window junkr, junkp;
-    WCoreWindow *frame;
-    WCoreWindow *onbotw[MAX_WINDOW_LEVELS], *last[MAX_WINDOW_LEVELS];
-    int level;
-    int i, c;
-    
-    if (!XQueryTree(dpy, scr->root_win, &junkr, &junkp, &windows, &nwindows)) {
-	wWarning(_("could not get window list!!"));
-	return;
-    } else {
-	c=0;
-	for (i=0; i<MAX_WINDOW_LEVELS; i++) {
-	    onbotw[i] = NULL;
-	    last[i] = NULL;
-	}
-	/* this ensures the correctness of the window level order
-	 * by using separate lists for each window level and then
-	 * concatenating all lists together */
-	for (i=0; i<nwindows; i++) {
-	    if (XFindContext(dpy, windows[i], wStackContext, (XPointer*)&frame)
-		==XCNOENT) {
-		continue;
-	    }
-	    if (!frame) continue;
-	    c++;
-	    level = levelToIndex(frame->stacking->window_level);
-	    if (onbotw[level])
-	      onbotw[level]->stacking->above = frame;
-	    else
-	      last[level] = frame;
-	    frame->stacking->under = onbotw[level];
-	    frame->stacking->above = NULL;
-	    onbotw[level] = frame;
-	}
-#ifdef DEBUG	
-	if (c!=scr->window_count) {
-	    puts("Found different number of windows than in window lists!!!");
-	    scr->window_count = c;
-	}
-#endif
-    }
-    scr->stacking_list = NULL;
-    /* now, just concatenate the lists */
-    for (i=0; i<MAX_WINDOW_LEVELS; i++) {
-	if (last[i]) {
-	    last[i]->stacking->under = scr->stacking_list;
-	    if (scr->stacking_list)
-	      scr->stacking_list->stacking->above = last[i];
-	    scr->stacking_list = onbotw[i];
-	}
-    }
-    CommitStacking(scr);
-}
-#else /* BUGGYSTACKING */
 void
 RemakeStackList(WScreen *scr)
 {
@@ -145,13 +66,13 @@ RemakeStackList(WScreen *scr)
     int i, c;
 
     if (!XQueryTree(dpy, scr->root_win, &junkr, &junkp, &windows, &nwindows)) {
-	wWarning(_("could not get window list!!"));
+	wwarning(_("could not get window list!!"));
 	return;
     } else {
 	for (i=0; i<MAX_WINDOW_LEVELS; i++) {
 	    scr->stacking_list[i] = NULL;
 	    onbotw[i] = NULL;
-	    scr->window_level_count[i] = 0;
+/*	    scr->window_level_count[i] = 0;*/
 	}
 	/* verify list integrity */
 	c=0;
@@ -162,15 +83,16 @@ RemakeStackList(WScreen *scr)
 	    }
 	    if (!frame) continue;
 	    c++;
-	    level = levelToIndex(frame->stacking->window_level);
+	    level = frame->stacking->window_level;
 	    if (onbotw[level])
 	      onbotw[level]->stacking->above = frame;
 	    frame->stacking->under = onbotw[level];
 	    frame->stacking->above = NULL;
 	    onbotw[level] = frame;
-	    scr->window_level_count[level]++;
-	}
-#ifdef DEBUG0
+/*	    scr->window_level_count[level]++;*/
+        }
+        XFree(windows);
+#ifdef DEBUG
 	if (c!=scr->window_count) {
 	    puts("Found different number of windows than in window lists!!!");
 	}
@@ -186,7 +108,6 @@ RemakeStackList(WScreen *scr)
     
     CommitStacking(scr);
 }
-#endif /* !BUGGYSTACKING */
 
 /*
  *----------------------------------------------------------------------
@@ -199,32 +120,6 @@ RemakeStackList(WScreen *scr)
  * 	Windows may be restacked.
  *---------------------------------------------------------------------- 
  */
-#ifdef BUGGYSTACKING 
-void
-CommitStacking(WScreen *scr)
-{
-    WCoreWindow *tmp;
-    int nwindows;
-    Window *windows;
-    int i;
-
-    nwindows = scr->window_count;
-    windows = wmalloc(sizeof(Window)*nwindows);
-    i=0;
-    tmp = scr->stacking_list;
-    while (tmp) {
-	if (i>=nwindows) {
-	    wWarning("Internal inconsistency! window_count incorrect.");
-	    free(windows);
-	    return;
-	}
-	windows[i++] = tmp->window;
-	tmp = tmp->stacking->under;
-    }
-    XRestackWindows(dpy, windows, i);
-    free(windows);
-}
-#else /* BUGGYSTACKING */
 void
 CommitStacking(WScreen *scr)
 {
@@ -239,7 +134,7 @@ CommitStacking(WScreen *scr)
     for (level=MAX_WINDOW_LEVELS-1; level>=0; level--) {
 	tmp = scr->stacking_list[level];
 	while (tmp) {
-#ifndef DEBUG
+#ifdef DEBUG
 	    if (i>=nwindows) {
 		puts("Internal inconsistency! window_count is incorrect!!!");
 		printf("window_count says %i windows\n", nwindows);
@@ -254,10 +149,7 @@ CommitStacking(WScreen *scr)
     XRestackWindows(dpy, windows, i);
     free(windows);
 }
-#endif /* !BUGGYSTACKING */
 
-
-#ifndef BUGGYSTACKING
 /*
  *----------------------------------------------------------------------
  * moveFrameToUnder--
@@ -279,7 +171,6 @@ moveFrameToUnder(WCoreWindow *under, WCoreWindow *frame)
     wins[1] = frame->window;
     XRestackWindows(dpy, wins, 2);
 }
-#endif /* !BUGGYSTACKING */
 
 /*
  *----------------------------------------------------------------------
@@ -295,61 +186,11 @@ moveFrameToUnder(WCoreWindow *under, WCoreWindow *frame)
  * 
  *---------------------------------------------------------------------- 
  */
-#ifdef BUGGYSTACKING
-void 
-wRaiseFrame(WCoreWindow *frame)
-{
-    WScreen *scr=frame->screen_ptr;
-    WCoreWindow *wlist=frame;
-
-    /* already on top */
-    if (frame->stacking->above == NULL
-	|| (frame->stacking->window_level
-	    !=wlist->stacking->above->stacking->window_level)) {
-	return;
-    }
-
-    while (wlist && (frame->stacking->window_level
-		     >= wlist->stacking->window_level)) {
-	wlist = wlist->stacking->above;
-    }
-    /* window is inserted before the point */
-    if (wlist==NULL) {
-	/* topmost window (last on the list) */
-	if (frame->stacking->under)
-	  frame->stacking->under->stacking->above = frame->stacking->above;
-	if (frame->stacking->above)
-	  frame->stacking->above->stacking->under = frame->stacking->under;
-
-	frame->stacking->above = NULL;
-	frame->stacking->under = scr->stacking_list;
-	scr->stacking_list->stacking->above=frame;
-	scr->stacking_list = frame;
-    } else if (frame != wlist) {
-	if (frame->stacking->under)
-	  frame->stacking->under->stacking->above = frame->stacking->above;
-	if (frame->stacking->above)
-	  frame->stacking->above->stacking->under = frame->stacking->under;
-	
-	frame->stacking->above = wlist;
-	frame->stacking->under = wlist->stacking->under;
-	if (wlist->stacking->under)
-	  wlist->stacking->under->stacking->above = frame;
-	wlist->stacking->under = frame;
-    }
-    /* try to optimize things a little */
-    if (frame->stacking->above == NULL) {
-	XRaiseWindow(dpy, frame->window);
-    } else {
-	CommitStacking(frame->screen_ptr);
-    }
-}
-#else /* !BUGGYSTACKING */
 void 
 wRaiseFrame(WCoreWindow *frame)
 {
     WCoreWindow *wlist=frame;
-    int level = levelToIndex(frame->stacking->window_level);
+    int level = frame->stacking->window_level;
     int i;
 
     /* already on top */
@@ -434,7 +275,6 @@ wRaiseFrame(WCoreWindow *frame)
 	moveFrameToUnder(frame->stacking->above, frame);
     }
 }
-#endif /* !BUGGYSTACKING */
 
 void
 wRaiseLowerFrame(WCoreWindow *frame)
@@ -448,58 +288,12 @@ wRaiseLowerFrame(WCoreWindow *frame)
 }
 
 
-#ifdef BUGGYSTACKING
 void 
 wLowerFrame(WCoreWindow *frame)
 {
     WScreen *scr=frame->screen_ptr;
     WCoreWindow *prev, *wlist=frame;
-
-    /* already in bottom */
-    if (wlist->stacking->under==NULL
-	|| (wlist->stacking->under->stacking->window_level
-	    !=frame->stacking->window_level)) {
-	return;
-    }
-    
-    /* it was the top window */
-    if (scr->stacking_list == frame) {
-	scr->stacking_list = frame->stacking->under;
-	scr->stacking_list->stacking->above = NULL;
-    } else {
-	if (frame->stacking->under)
-	  frame->stacking->under->stacking->above = frame->stacking->above;
-	if (frame->stacking->above)
-	  frame->stacking->above->stacking->under = frame->stacking->under;
-    }
-    wlist = scr->stacking_list;
-    prev = wlist;
-    while (wlist && (frame->stacking->window_level
-		     <= wlist->stacking->window_level)
-	   && frame->stacking->child_of!=wlist) {
-	prev = wlist;
-	wlist = wlist->stacking->under;
-    }
-    frame->stacking->above = prev;
-    frame->stacking->under = prev->stacking->under;
-    if (prev->stacking->under)
-      prev->stacking->under->stacking->above = frame;
-    prev->stacking->under = frame;
-
-    /* try to optimize things a little */
-    if (frame->stacking->under == NULL) {
-	XLowerWindow(dpy, frame->window);
-    } else {
-	CommitStacking(scr);
-    }
-}
-#else /* BUGGYSTACKING */
-void 
-wLowerFrame(WCoreWindow *frame)
-{
-    WScreen *scr=frame->screen_ptr;
-    WCoreWindow *prev, *wlist=frame;
-    int level = levelToIndex(frame->stacking->window_level);
+    int level = frame->stacking->window_level;
     int i;
 
     /* already in bottom */
@@ -569,7 +363,6 @@ wLowerFrame(WCoreWindow *frame)
 	moveFrameToUnder(frame->stacking->above, frame);
     }
 }
-#endif /* !BUGGYSTACKING */
 
 
 /*
@@ -589,10 +382,10 @@ void
 AddToStackList(WCoreWindow *frame)
 {
     WCoreWindow *prev, *tmpw, *wlist;
-    int index = levelToIndex(frame->stacking->window_level);
+    int index = frame->stacking->window_level;
 
     frame->screen_ptr->window_count++;
-    frame->screen_ptr->window_level_count[index]++;
+/*    frame->screen_ptr->window_level_count[index]++;*/
     XSaveContext(dpy, frame->window, wStackContext, (XPointer)frame);
     tmpw = frame->screen_ptr->stacking_list[index];
     if (!tmpw) {
@@ -640,25 +433,30 @@ AddToStackList(WCoreWindow *frame)
 /*
  *----------------------------------------------------------------------
  * MoveInStackListAbove--
- * 	Moves the frame above "next". Note that both frame and "next"
- * must be in the same window level.
+ * 	Moves the frame above "next".
  * 
  * Returns:
  * 	None
  * 
  * Side effects:
  * 	Stacking order may be changed.
+ *      Window level for frame may be changed.
  *---------------------------------------------------------------------- 
  */
 void
-MoveInStackListAbove(WCoreWindow *frame, WCoreWindow *next)
+MoveInStackListAbove(WCoreWindow *next, WCoreWindow *frame)
 {
     WCoreWindow *tmpw;
-    int index = levelToIndex(frame->stacking->window_level);
+    int index;
 
     if (!next || frame->stacking->under == next)
         return;
 
+    if (frame->stacking->window_level != next->stacking->window_level)
+        ChangeStackingLevel(frame, next->stacking->window_level);
+
+    index = frame->stacking->window_level;
+
     tmpw = frame->screen_ptr->stacking_list[index];
     if (tmpw == frame)
         frame->screen_ptr->stacking_list[index] = frame->stacking->under;
@@ -673,18 +471,83 @@ MoveInStackListAbove(WCoreWindow *frame, WCoreWindow *next)
     next->stacking->above = frame;
     if (tmpw == next)
         frame->screen_ptr->stacking_list[index] = frame;
-    CommitStacking(frame->screen_ptr);
+
+    /* try to optimize things a little */
+    if (frame->stacking->above == NULL) {
+        WCoreWindow *above=NULL;
+        int i;
+
+        for (i=index+1; i<MAX_WINDOW_LEVELS; i++) {
+            if (frame->screen_ptr->stacking_list[i]!=NULL) {
+                /* can't optimize */
+                above = frame->screen_ptr->stacking_list[i];
+                while (above->stacking->under)
+                    above = above->stacking->under;
+                break;
+            }
+        }
+        if (!above) {
+            XRaiseWindow(dpy, frame->window);
+        } else {
+            moveFrameToUnder(above, frame);
+        }
+    } else {
+        moveFrameToUnder(frame->stacking->above, frame);
+    }
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ * MoveInStackListUnder--
+ * 	Moves the frame under "prev".
+ * 
+ * Returns:
+ * 	None
+ * 
+ * Side effects:
+ * 	Stacking order may be changed.
+ *      Window level for frame may be changed.
+ *---------------------------------------------------------------------- 
+ */
+void
+MoveInStackListUnder(WCoreWindow *prev, WCoreWindow *frame)
+{
+    WCoreWindow *tmpw;
+    int index;
+
+    if (!prev || frame->stacking->above == prev)
+        return;
+
+    if (frame->stacking->window_level != prev->stacking->window_level)
+        ChangeStackingLevel(frame, prev->stacking->window_level);
+
+    index = frame->stacking->window_level;
+
+    tmpw = frame->screen_ptr->stacking_list[index];
+    if (tmpw == frame)
+        frame->screen_ptr->stacking_list[index] = frame->stacking->under;
+    if (frame->stacking->under)
+        frame->stacking->under->stacking->above = frame->stacking->above;
+    if (frame->stacking->above)
+        frame->stacking->above->stacking->under = frame->stacking->under;
+    if (prev->stacking->under)
+        prev->stacking->under->stacking->above = frame;
+    frame->stacking->above = prev;
+    frame->stacking->under = prev->stacking->under;
+    prev->stacking->under = frame;
+    moveFrameToUnder(prev, frame);
 }
 
 
 void
 RemoveFromStackList(WCoreWindow *frame)
 {
-    int index = levelToIndex(frame->stacking->window_level);
+    int index = frame->stacking->window_level;
 
     if (XDeleteContext(dpy, frame->window, wStackContext)==XCNOENT) {
 #ifdef DEBUG0
-	wWarning("RemoveFromStackingList(): window not in list ");
+	wwarning("RemoveFromStackingList(): window not in list ");
 #endif
 	return;
     }
@@ -697,7 +560,7 @@ RemoveFromStackList(WCoreWindow *frame)
       frame->screen_ptr->stacking_list[index] = frame->stacking->under;
 
     frame->screen_ptr->window_count--;
-    frame->screen_ptr->window_level_count[index]--;
+/*    frame->screen_ptr->window_level_count[index]--;*/
 }
 
 
