@@ -30,7 +30,6 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "generic/wwmlib.h"
 #include "WindowMaker.h"
 #include "funcs.h"
 #include "text.h"
@@ -187,7 +186,7 @@ textXtoPos( WTextInput *wtext, int x )
     if( x < 0 )
       break;
     else
-      x -= MyTextWidth( wtext->font->font, &(wtext->text.txt[pos]), 1 );
+      x -= wTextWidth( wtext->font->font, &(wtext->text.txt[pos]), 1 );
     }
   
   return pos;
@@ -205,27 +204,27 @@ WTextInput *
 wTextCreate( WCoreWindow *core, int x, int y, int width, int height )
   {
   WTextInput *wtext;
-  
+
   ENTER("wTextCreate");
-    
+
   wtext = wmalloc(sizeof(WTextInput));
   wtext->core = wCoreCreate( core, x, y, width, height );
   wtext->core->descriptor.handle_anything = &textEventHandler; 
   wtext->core->descriptor.handle_expose = &handleExpose;     
   wtext->core->descriptor.parent_type = WCLASS_TEXT_INPUT;
   wtext->core->descriptor.parent = wtext;
-  
+
   wtext->font = core->screen_ptr->menu_entry_font;
 
   XDefineCursor( dpy, wtext->core->window, wCursor[WCUR_TEXT] );
-      
+
   /* setup the text: */
   wtext->text.txt      = (char *)wmalloc(sizeof(char));
   wtext->text.txt[0]   = '\0';
   wtext->text.length   = 0;
   wtext->text.startPos = 0;
   wtext->text.endPos   = 0;
-  
+
     {
     XGCValues gcv;
 
@@ -236,11 +235,14 @@ wTextCreate( WCoreWindow *core, int x, int y, int width, int height )
 #endif
     gcv.line_width = 1;
     gcv.function   = GXcopy;
-    
+
     wtext->gc = XCreateGC( dpy, wtext->core->window,
-                            (GCForeground|GCBackground|
-                             GCFunction|GCFont|GCLineWidth),
-                            &gcv );
+                           (GCForeground|GCBackground|
+#ifndef I18N_MB
+                            GCFont|
+#endif
+                            GCFunction|GCLineWidth),
+                           &gcv );
 
     /* set up the regular context */
     gcv.foreground = core->screen_ptr->black_pixel;
@@ -250,20 +252,26 @@ wTextCreate( WCoreWindow *core, int x, int y, int width, int height )
 #endif
     gcv.line_width = 1;
     gcv.function   = GXcopy;
-    
+
     wtext->regGC = XCreateGC( dpy, wtext->core->window,
-                            (GCForeground|GCBackground|
-                             GCFunction|GCFont|GCLineWidth),
-                            &gcv );
-    
+                              (GCForeground|GCBackground|
+#ifndef I18N_MB
+                               GCFont|
+#endif
+                               GCFunction|GCLineWidth),
+                              &gcv );
+
     /* set up the inverted context */
     gcv.function   = GXcopyInverted;
-    
+
     wtext->invGC = XCreateGC( dpy, wtext->core->window,
-                            (GCForeground|GCBackground|
-                             GCFunction|GCFont|GCLineWidth),
-                            &gcv );
-    
+                              (GCForeground|GCBackground|
+#ifndef I18N_MB
+                               GCFont|
+#endif
+                               GCFunction|GCLineWidth),
+                              &gcv );
+
     /* and set the background! */
     XSetWindowBackground( dpy, wtext->core->window, gcv.background );
     }
@@ -298,6 +306,7 @@ wTextDestroy( WTextInput *wtext )
 #if 0
   if (wtext->magic)
       wDeleteTimerHandler(wtext->magic);
+  wtext->magic = NULL;
 #endif
   XFreeGC( dpy, wtext->gc );
   XFreeGC( dpy, wtext->regGC );
@@ -349,13 +358,8 @@ textRefresh( WTextInput *wtext )
                 x1, y1, x2-x1, y2-y1 );
 
   /* Draw the text normally */
-#ifndef I18N_MB
-  XDrawString( dpy, wtext->core->window, wtext->regGC, x1, y2+TEXT_SHIFT,
-             ptr, wtext->text.length );
-#else
-  XmbDrawString(dpy, wtext->core->window, wtext->font->font, wtext->regGC, 
+  wDrawString(wtext->core->window, wtext->font, wtext->regGC, 
 		x1, y2+TEXT_SHIFT, ptr, wtext->text.length);
-#endif
   
   /* Draw the selected text */
   if( wtext->text.startPos != wtext->text.endPos )
@@ -374,10 +378,10 @@ textRefresh( WTextInput *wtext )
       }
     
     /* x1,y1 is now the upper-left of the selected area */
-    x1 += MyTextWidth( wtext->font->font, ptr, sp );
+    x1 += wTextWidth( wtext->font->font, ptr, sp );
     /* and x2,y2 is the lower-right of the selected area */
     ptr += sp * sizeof(char);
-    x2 = x1 +MyTextWidth( wtext->font->font, ptr, (ep - sp) );
+    x2 = x1 +wTextWidth( wtext->font->font, ptr, (ep - sp) );
     /* Fill in the area  where the selected text will go:     *
      * use the regGC to draw the rectangle, becuase then it   *
      * will be the color of the non-selected text             */
@@ -386,17 +390,12 @@ textRefresh( WTextInput *wtext )
     
     /* Draw the selected text... use invGC so it will be the
      * opposite color as the filled rectangle */
-#ifndef I18N_MB
-    XDrawString( dpy, wtext->core->window, wtext->invGC, x1, y2+TEXT_SHIFT,
-               ptr, (ep - sp) );
-#else
-    XmbDrawString(dpy, wtext->core->window, wtext->font->font, wtext->invGC, 
+    wDrawString(wtext->core->window, wtext->font, wtext->invGC, 
 	 	  x1, y2+TEXT_SHIFT, ptr, (ep - sp));
-#endif
     }
   
   /* And draw a quick little line for the cursor position */
-  x1 = MyTextWidth( wtext->font->font, wtext->text.txt, wtext->text.endPos )
+  x1 = wTextWidth( wtext->font->font, wtext->text.txt, wtext->text.endPos )
        + wtext->xOffset;
   XDrawLine( dpy, wtext->core->window, wtext->regGC, x1, 2, x1, 
 	     wtext->core->height - 3 );
@@ -465,8 +464,8 @@ wTextPutText( WTextInput *wtext, char *txt )
   if( wtext->text.txt != NULL )
     free( wtext->text.txt );
   
-  wtext->text.txt = (char *)malloc(length*sizeof(char));
-  strcpy( wtext->text.txt, txt );
+  wtext->text.txt = (char *)wmalloc((length+1)*sizeof(char));
+  strcpy(wtext->text.txt, txt );
   wtext->text.length   = length;
   /* By default No text is selected, and the cursor is at the end */
   wtext->text.startPos = length;
@@ -506,7 +505,7 @@ textInsert( WTextInput *wtext, char *txt )
   txtLen = strlen(txt);
   newLen = wtext->text.length + txtLen - (ep - sp) + 1;
   
-  newTxt = (char *)malloc(newLen*sizeof(char *));
+  newTxt = (char *)malloc(newLen*sizeof(char));
   
   /* copy the old text up to sp */
   for( i=0; i<sp; i++ )
@@ -572,7 +571,7 @@ blink(void *data)
 	gc = wtext->invGC;
 	wtext->blink_on = 1;
     }
-    x = MyTextWidth( wtext->font->font, wtext->text.txt, wtext->text.endPos )
+    x = wTextWidth( wtext->font->font, wtext->text.txt, wtext->text.endPos )
 	+ wtext->xOffset;
     XDrawLine( dpy, wtext->core->window, gc, x, 2, x, wtext->core->height-3);
 
@@ -661,7 +660,7 @@ textEventHandler( WObjDescriptor *desc, XEvent *event )
   if( handled )
     textRefresh(wtext);
   else
-    DispatchEvent(event);
+    WMHandleEvent(event);
   
   return;
   }
