@@ -1,7 +1,7 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 
@@ -29,11 +30,8 @@
 #define WREL_RAISED	0
 #define WREL_SUNKEN	1
 #define WREL_FLAT	2
-#define WREL_TITLEBAR	3
 #define WREL_ICON	4
-#define WREL_MENUTITLE	5
 #define WREL_MENUENTRY	6
-#define WREL_MENUBUTTON 7
 
 /* texture types */
 #define WREL_BORDER_MASK	1
@@ -50,7 +48,7 @@
 /* pixmap subtypes */
 #define WTP_TILE	2
 #define WTP_SCALE	4
-
+#define WTP_CENTER	6
 
 /*
  * (solid <color>)
@@ -153,6 +151,7 @@ void wTexturePaintTitlebar(struct WWindow *wwin, WTexture *texture, Pixmap *tdat
 
 #define FREE_PIXMAP(p) if ((p)!=None) XFreePixmap(dpy, (p)), (p)=None
 
-void wDrawBevel(WCoreWindow *core, WTexSolid *texture, int relief);
+void wDrawBevel(Drawable d, unsigned width, unsigned height,
+		WTexSolid *texture, int relief);
 
 #endif
