@@ -1,8 +1,8 @@
 /* workspace.c- Workspace management
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
- *  Copyright (c) 1997 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  USA.
  */
 
 #ifndef WMWORKSPACE_H_
@@ -26,26 +27,24 @@
 
 typedef struct WWorkspace {
     char *name;
-#ifdef DOCK
-    struct WDock *fiend;
-#endif
-#ifdef PAGER
-    struct WPagerView *view;
-#endif
+    struct WDock *clip;
 } WWorkspace;
 
 void wWorkspaceMake(WScreen *scr, int count);
 int wWorkspaceNew(WScreen *scr);
 void wWorkspaceDeleteLast(WScreen *scr);
 void wWorkspaceChange(WScreen *scr, int workspace);
+void wWorkspaceForceChange(WScreen *scr, int workspace);
 
-WMenu *wWorkspaceMenuMake(WScreen *scr);
+
+WMenu *wWorkspaceMenuMake(WScreen *scr, Bool titled);
 void wWorkspaceMenuUpdate(WScreen *scr, WMenu *menu);
 
 void wWorkspaceMenuEdit(WScreen *scr);
 
-void wWorkspaceSaveState(WScreen *scr);
+void wWorkspaceSaveState(WScreen *scr, proplist_t old_state);
 void wWorkspaceRestoreState(WScreen *scr);
 
+void wWorkspaceRename(WScreen *scr, int workspace, char *name);
 
 #endif
