@@ -1,6 +1,6 @@
 
 #include <X11/Xlib.h>
-#include <wraster.h>
+#include "wraster.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include "tile.xpm"
@@ -31,16 +31,16 @@ void main(int argc, char **argv)
 	img = RLoadImage(ctx, argv[1], argc>2 ? atol(argv[2]) : 0);
 
     if (!img) {
-	puts(RErrorString);
+	puts(RMessageForError(RErrorCode));
 	exit(1);
     }
     if (!RConvertImage(ctx, img, &pix)) {
-	puts(RErrorString);
+	puts(RMessageForError(RErrorCode));
 	exit(1);
     }
 
     win = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy), 10, 10, img->width,
-					img->height, 0, 0, 0);
+			      img->height, 0, 0, 0);
     XSelectInput(dpy, win, ExposureMask);
     XMapRaised(dpy, win);
     XSync(dpy, False); 
@@ -57,5 +57,5 @@ void main(int argc, char **argv)
     }
     XFlush(dpy);
 
-    getchar();  
+    getchar();
 }
